import datetime as dt

from django.test import TestCase
from django.utils.timezone import now

from allianceauth.authentication.task_statistics.counters import (
    dashboard_results,
    succeeded_tasks,
    retried_tasks,
    failed_tasks,
)


class TestDashboardResults(TestCase):
    def test_should_return_counts_for_given_timeframe_only(self):
        # given
        earliest_task = now() - dt.timedelta(minutes=15)
        succeeded_tasks.clear()
        succeeded_tasks.add(now() - dt.timedelta(hours=1, seconds=1))
        succeeded_tasks.add(earliest_task)
        succeeded_tasks.add()
        succeeded_tasks.add()
        retried_tasks.clear()
        retried_tasks.add(now() - dt.timedelta(hours=1, seconds=1))
        retried_tasks.add(now() - dt.timedelta(seconds=30))
        retried_tasks.add()
        failed_tasks.clear()
        failed_tasks.add(now() - dt.timedelta(hours=1, seconds=1))
        failed_tasks.add()
        # when
        results = dashboard_results(hours=1)
        # then
        self.assertEqual(results.succeeded, 3)
        self.assertEqual(results.retried, 2)
        self.assertEqual(results.failed, 1)
        self.assertEqual(results.total, 6)
        self.assertEqual(results.earliest_task, earliest_task)

    def test_should_work_with_no_data(self):
        # given
        succeeded_tasks.clear()
        retried_tasks.clear()
        failed_tasks.clear()
        # when
        results = dashboard_results(hours=1)
        # then
        self.assertEqual(results.succeeded, 0)
        self.assertEqual(results.retried, 0)
        self.assertEqual(results.failed, 0)
        self.assertEqual(results.total, 0)
        self.assertIsNone(results.earliest_task)



import datetime
import logging

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.contrib.auth.models import Group
from django.shortcuts import render, redirect
from django.utils.translation import gettext_lazy as _

from allianceauth.services.forms import ServicePasswordForm

from .forms import JabberBroadcastForm
from .manager import OpenfireManager, PingBotException
from .models import OpenfireUser
from .tasks import OpenfireTasks

logger = logging.getLogger(__name__)

ACCESS_PERM = 'openfire.access_openfire'


@login_required
@permission_required(ACCESS_PERM)
def activate_jabber(request):
    logger.debug("activate_jabber called by user %s" % request.user)
    character = request.user.profile.main_character
    logger.debug(f"Adding jabber user for user {request.user} with main character {character}")
    info = OpenfireManager.add_user(OpenfireTasks.get_username(request.user))
    # If our username is blank means we already had a user
    if info[0] != "":
        OpenfireUser.objects.update_or_create(user=request.user, defaults={'username': info[0]})
        logger.debug("Updated authserviceinfo for user %s with jabber credentials. Updating groups." % request.user)
        OpenfireTasks.update_groups.delay(request.user.pk)
        logger.info("Successfully activated jabber for user %s" % request.user)
        messages.success(request, _('Activated jabber account.'))
        credentials = {
            'username': info[0],
            'password': info[1],
        }
        return render(request, 'services/service_credentials.html', context={'credentials': credentials, 'service': 'Jabber'})
    else:
        logger.error("Unsuccessful attempt to activate jabber for user %s" % request.user)
        messages.error(request, _('An error occurred while processing your jabber account.'))
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def deactivate_jabber(request):
    logger.debug("deactivate_jabber called by user %s" % request.user)
    if OpenfireTasks.has_account(request.user) and OpenfireTasks.delete_user(request.user):
        logger.info("Successfully deactivated jabber for user %s" % request.user)
        messages.success(request, 'Deactivated jabber account.')
    else:
        logger.error("Unsuccessful attempt to deactivate jabber for user %s" % request.user)
        messages.error(request, _('An error occurred while processing your jabber account.'))
    return redirect("services:services")


@login_required
@permission_required(ACCESS_PERM)
def reset_jabber_password(request):
    logger.debug("reset_jabber_password called by user %s" % request.user)
    if OpenfireTasks.has_account(request.user):
        result = OpenfireManager.update_user_pass(request.user.openfire.username)
        # If our username is blank means we failed
        if result != "":
            logger.info("Successfully reset jabber password for user %s" % request.user)
            messages.success(request, _('Reset jabber password.'))
            credentials = {
                'username': request.user.openfire.username,
                'password': result,
            }
            return render(request, 'services/service_credentials.html', context={'credentials': credentials, 'service': 'Jabber'})
    logger.error("Unsuccessful attempt to reset jabber for user %s" % request.user)
    messages.error(request, _('An error occurred while processing your jabber account.'))
    return redirect("services:services")


@login_required
@permission_required('auth.jabber_broadcast')
def jabber_broadcast_view(request):
    logger.debug("jabber_broadcast_view called by user %s" % request.user)
    allchoices = []
    if request.user.has_perm('auth.jabber_broadcast_all'):
        allchoices.append(('all', 'all'))
        for g in Group.objects.all():
            allchoices.append((str(g.name), str(g.name)))
    else:
        for g in request.user.groups.all():
            allchoices.append((str(g.name), str(g.name)))
    if request.method == 'POST':
        form = JabberBroadcastForm(request.POST)
        form.fields['group'].choices = allchoices
        logger.debug("Received POST request containing form, valid: %s" % form.is_valid())
        if form.is_valid():
            main_char = request.user.profile.main_character
            logger.debug(f"Processing jabber broadcast for user {request.user} with main character {main_char}")
            try:
                if main_char is not None:
                    message_to_send = form.cleaned_data['message'] + "\n##### SENT BY: " + "[" + main_char.corporation_ticker + "]" + \
                                        main_char.character_name + " TO: " + \
                                        form.cleaned_data['group'] + " WHEN: " + datetime.datetime.utcnow().strftime(
                                            "%Y-%m-%d %H:%M:%S") + " #####\n##### Replies are NOT monitored #####\n"
                    group_to_send = form.cleaned_data['group']

                else:
                    message_to_send = form.cleaned_data['message'] + "\n##### SENT BY: " + "No character but can send pings?" + " TO: " + \
                                        form.cleaned_data['group'] + " WHEN: " + datetime.datetime.utcnow().strftime(
                                            "%Y-%m-%d %H:%M:%S") + " #####\n##### Replies are NOT monitored #####\n"
                    group_to_send = form.cleaned_data['group']

                OpenfireManager.send_broadcast_message(group_to_send, message_to_send)

                messages.success(request, _('Sent jabber broadcast to %s' % group_to_send))
                logger.info("Sent jabber broadcast on behalf of user %s" % request.user)
            except PingBotException as e:
                messages.error(request, e)

    else:
        form = JabberBroadcastForm()
        form.fields['group'].choices = allchoices
        logger.debug("Generated broadcast form for user {} containing {} groups".format(
            request.user, len(form.fields['group'].choices)))

    context = {'form': form}
    return render(request, 'services/openfire/broadcast.html', context=context)


@login_required
@permission_required(ACCESS_PERM)
def set_jabber_password(request):
    logger.debug("set_jabber_password called by user %s" % request.user)
    if request.method == 'POST':
        logger.debug("Received POST request with form.")
        form = ServicePasswordForm(request.POST)
        logger.debug("Form is valid: %s" % form.is_valid())
        if form.is_valid() and OpenfireTasks.has_account(request.user):
            password = form.cleaned_data['password']
            logger.debug("Form contains password of length %s" % len(password))
            result = OpenfireManager.update_user_pass(request.user.openfire.username, password=password)
            if result != "":
                logger.info("Successfully set jabber password for user %s" % request.user)
                messages.success(request, _('Set jabber password.'))
            else:
                logger.error("Failed to install custom jabber password for user %s" % request.user)
                messages.error(request, _('An error occurred while processing your jabber account.'))
            return redirect("services:services")
    else:
        logger.debug("Request is not type POST - providing empty form.")
        form = ServicePasswordForm()

    logger.debug("Rendering form for user %s" % request.user)
    context = {'form': form, 'service': 'Jabber'}
    return render(request, 'services/service_password.html', context=context)



import logging

from django.db import transaction
from django.db.models import Manager, QuerySet, Q

from allianceauth.eveonline.models import EveCharacter

logger = logging.getLogger(__name__)


def available_states_query(character):
    query = Q(public=True)
    if character.character_id:
        query |= Q(member_characters__character_id=character.character_id)
    if character.corporation_id:
        query |= Q(member_corporations__corporation_id=character.corporation_id)
    if character.alliance_id:
        query |= Q(member_alliances__alliance_id=character.alliance_id)
    if character.faction_id:
        query |= Q(member_factions__faction_id=character.faction_id)
    return query


class CharacterOwnershipManager(Manager):
    def create_by_token(self, token):
        if not EveCharacter.objects.filter(character_id=token.character_id).exists():
            EveCharacter.objects.create_character(token.character_id)
        return self.create(character=EveCharacter.objects.get(character_id=token.character_id), user=token.user, owner_hash=token.character_owner_hash)


class StateQuerySet(QuerySet):
    def available_to_character(self, character):
        return self.filter(available_states_query(character))

    def available_to_user(self, user):
        if user.profile.main_character:
            return self.available_to_character(user.profile.main_character)
        else:
            return self.none()

    def get_for_user(self, user):
        states = self.available_to_user(user)
        if states.exists():
            return states[0]
        else:
            from allianceauth.authentication.models import get_guest_state
            return get_guest_state()

    def delete(self):
        with transaction.atomic():
            for state in self:
                for profile in state.userprofile_set.all():
                    profile.assign_state(state=self.model.objects.exclude(pk=state.pk).get_for_user(profile.user))
        super().delete()


class StateManager(Manager):
    def get_queryset(self):
        return StateQuerySet(self.model, using=self._db)

    def available_to_character(self, character):
        return self.get_queryset().available_to_character(character)

    def available_to_user(self, user):
        return self.get_queryset().available_to_user(user)

    def get_for_character(self, character):
        states = self.get_queryset().available_to_character(character)
        if states.exists():
            return states[0]
        else:
            from allianceauth.authentication.models import get_guest_state
            return get_guest_state()

    def get_for_user(self, user):
        return self.get_queryset().get_for_user(user)



from django.db import migrations, models
from ..auth_hooks import MumbleService
from allianceauth.services.hooks import NameFormatter

def fwd_func(apps, schema_editor):
    MumbleUser = apps.get_model("mumble", "MumbleUser")
    db_alias = schema_editor.connection.alias
    all_users = MumbleUser.objects.using(db_alias).all()
    for user in all_users:
        display_name = NameFormatter(MumbleService(), user.user).format_name()
        user.display_name = display_name
        user.save()

def rev_func(apps, schema_editor):
    MumbleUser = apps.get_model("mumble", "MumbleUser")
    db_alias = schema_editor.connection.alias
    all_users = MumbleUser.objects.using(db_alias).all()
    for user in all_users:
        user.display_name = None
        user.save()


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0008_mumbleuser_display_name'),
    ]

    operations = [
        migrations.RunPython(fwd_func, rev_func),
        migrations.AlterField(
            model_name='mumbleuser',
            name='display_name',
            field=models.CharField(max_length=254, unique=True),
            preserve_default=False,
        ),
    ]



import logging
import os


def set_logger(logger_name: str, name: str) -> object:
    """set logger for current test module

    Args:
    - logger: current logger object
    - name: name of current module, e.g. __file__

    Returns:
    - amended logger
    """

    # reconfigure logger so we get logging from tested module
    f_format = logging.Formatter(
        '%(asctime)s - %(levelname)s - %(module)s:%(funcName)s - %(message)s'
    )
    f_handler = logging.FileHandler(
        f'{os.path.splitext(name)[0]}.log',
        'w+'
    )
    f_handler.setFormatter(f_format)
    logger = logging.getLogger(logger_name)
    logger.level = logging.DEBUG
    logger.addHandler(f_handler)
    logger.propagate = False
    return logger



from django.contrib import admin

from .models import AnalyticsIdentifier, AnalyticsPath, AnalyticsTokens


@admin.register(AnalyticsIdentifier)
class AnalyticsIdentifierAdmin(admin.ModelAdmin):
    search_fields = ['identifier', ]
    list_display = ('identifier',)


@admin.register(AnalyticsTokens)
class AnalyticsTokensAdmin(admin.ModelAdmin):
    search_fields = ['name', ]
    list_display = ('name', 'type',)


@admin.register(AnalyticsPath)
class AnalyticsPathAdmin(admin.ModelAdmin):
    search_fields = ['ignore_path', ]
    list_display = ('ignore_path',)



from django.test import TestCase
from allianceauth.tests.auth_utils import AuthUtils
from allianceauth.eveonline.models import EveAllianceInfo, EveCorporationInfo, EveCharacter
from ..models import NameFormatConfig
from ..hooks import NameFormatter
from ..modules.example.auth_hooks import ExampleService


class NameFormatterTestCase(TestCase):
    def setUp(self):
        self.member = AuthUtils.create_user('auth_member', disconnect_signals=True)

        self.alliance = EveAllianceInfo.objects.create(
            alliance_id='3456',
            alliance_name='alliance name',
            alliance_ticker='TIKR',
            executor_corp_id='2345',
        )

        self.corp = EveCorporationInfo.objects.create(
            corporation_id='2345',
            corporation_name='corp name',
            corporation_ticker='TIKK',
            member_count=10,
            alliance=self.alliance,
        )

        self.char = EveCharacter.objects.create(
            character_id='1234',
            character_name='test character',
            corporation_id='2345',
            corporation_name='test corp',
            corporation_ticker='TIKK',
            alliance_id='3456',
            alliance_name='alliance name',
            alliance_ticker='TIKR',
        )
        self.member.profile.main_character = self.char
        self.member.profile.save()

    def test_formatter_prop(self):
        config = NameFormatConfig.objects.create(
            service_name='example',
            default_to_username=False,
            format='{character_name}',
        )

        config.states.add(self.member.profile.state)

        formatter = NameFormatter(ExampleService(), self.member)

        self.assertEqual(config, formatter.formatter_config)

    def test_default_formatter(self):

        formatter = NameFormatter(ExampleService(), self.member)

        self.assertEqual(NameFormatter.DEFAULT_FORMAT, formatter.default_formatter)
        # Test the default is returned when the service has no default
        self.assertEqual(NameFormatter.DEFAULT_FORMAT, formatter.string_formatter)

    def test_get_format_data(self):
        config = NameFormatConfig.objects.create(
            service_name='example',
            default_to_username=False,
            format='{alliance_ticker}',  # Ensures that alliance_ticker is filled
        )
        config.states.add(self.member.profile.state)

        formatter = NameFormatter(ExampleService(), self.member)

        result = formatter.get_format_data()

        self.assertIn('character_name', result)
        self.assertEqual(result['character_name'], self.char.character_name)
        self.assertIn('character_id', result)
        self.assertEqual(result['character_id'], self.char.character_id)
        self.assertIn('corp_name', result)
        self.assertEqual(result['corp_name'], self.char.corporation_name)
        self.assertIn('corp_id', result)
        self.assertEqual(result['corp_id'], self.char.corporation_id)
        self.assertIn('corp_ticker', result)
        self.assertEqual(result['corp_ticker'], self.char.corporation_ticker)
        self.assertIn('alliance_name', result)
        self.assertEqual(result['alliance_name'], self.char.alliance_name)
        self.assertIn('alliance_ticker', result)
        self.assertEqual(result['alliance_ticker'], self.char.alliance_ticker)
        self.assertIn('alliance_id', result)
        self.assertEqual(result['alliance_id'], self.char.alliance_id)
        self.assertIn('username', result)
        self.assertEqual(result['username'], self.member.username)
        self.assertIn('alliance_or_corp_name', result)
        self.assertEqual(result['alliance_or_corp_name'], self.char.alliance_name)
        self.assertIn('alliance_or_corp_ticker', result)
        self.assertEqual(result['alliance_or_corp_ticker'], self.char.alliance_ticker)

    def test_format_name(self):
        config = NameFormatConfig.objects.create(
            service_name='example',
            default_to_username=False,
            format='{character_id} test {username}',
        )
        config.states.add(self.member.profile.state)
        formatter = NameFormatter(ExampleService(), self.member)

        result = formatter.format_name()

        self.assertEqual('1234 test auth_member', result)



import json

from unittest.mock import patch, Mock

from django.test import TestCase, RequestFactory
from django.urls import reverse

from allianceauth.tests.auth_utils import AuthUtils

from ..views import user_notifications_count


MODULE_PATH = 'allianceauth.notifications.views'


class TestViews(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.user = AuthUtils.create_user('magic_mike')
        AuthUtils.add_main_character(
            cls.user,
            'Magic Mike',
            '1',
            corp_id='2',
            corp_name='Pole Riders',
            corp_ticker='PRIDE',
            alliance_id='3',
            alliance_name='RIDERS'
        )
        cls.factory = RequestFactory()

    @patch(MODULE_PATH + '.Notification.objects.user_unread_count')
    def test_user_notifications_count(self, mock_user_unread_count):
        unread_count = 42
        user_pk = 3
        mock_user_unread_count.return_value = unread_count

        request = self.factory.get(
            reverse('notifications:user_notifications_count', args=[user_pk])
        )
        request.user = self.user

        response = user_notifications_count(request, user_pk)
        self.assertEqual(response.status_code, 200)
        self.assertTrue(mock_user_unread_count.called)
        expected = {'unread_count': unread_count}
        result = json.loads(response.content.decode(response.charset))
        self.assertDictEqual(result, expected)



import os

from bravado.exception import HTTPError
from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required, user_passes_test
from django.core.exceptions import PermissionDenied, ObjectDoesNotExist
from django.db import IntegrityError
from django.shortcuts import render, redirect, get_object_or_404
from django.utils.translation import ugettext_lazy as _
from esi.decorators import token_required
from allianceauth.eveonline.models import EveCharacter, EveCorporationInfo

from .models import CorpStats, CorpMember

SWAGGER_SPEC_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'swagger.json')
"""
Swagger spec operations:

get_characters_character_id
"""


def access_corpstats_test(user):
    return user.has_perm('corputils.view_corp_corpstats') or user.has_perm(
        'corputils.view_alliance_corpstats') or user.has_perm('corputils.view_state_corpstats') or user.has_perm(
        'corputils.add_corpstats')


@login_required
@user_passes_test(access_corpstats_test)
@permission_required('corputils.add_corpstats')
@token_required(scopes='esi-corporations.read_corporation_membership.v1')
def corpstats_add(request, token):
    try:
        if EveCharacter.objects.filter(character_id=token.character_id).exists():
            corp_id = EveCharacter.objects.get(character_id=token.character_id).corporation_id
        else:
            corp_id = \
                token.get_esi_client(spec_file=SWAGGER_SPEC_PATH).Character.get_characters_character_id(
                    character_id=token.character_id).result()['corporation_id']
        try:
            corp = EveCorporationInfo.objects.get(corporation_id=corp_id)
        except EveCorporationInfo.DoesNotExist:
            corp = EveCorporationInfo.objects.create_corporation(corp_id)
        cs = CorpStats.objects.create(token=token, corp=corp)
        try:
            cs.update()
        except HTTPError as e:
            messages.error(request, str(e))
        assert cs.pk  # ensure update was successful
        if CorpStats.objects.filter(pk=cs.pk).visible_to(request.user).exists():
            return redirect('corputils:view_corp', corp_id=corp.corporation_id)
    except IntegrityError:
        messages.error(request, _('Selected corp already has a statistics module.'))
    except AssertionError:
        messages.error(request, _('Failed to gather corporation statistics with selected token.'))
    return redirect('corputils:view')


@login_required
@user_passes_test(access_corpstats_test)
def corpstats_view(request, corp_id=None):
    corpstats = None

    # get requested model
    if corp_id:
        corp = get_object_or_404(EveCorporationInfo, corporation_id=corp_id)
        corpstats = get_object_or_404(CorpStats, corp=corp)

    # get available models
    available = CorpStats.objects.visible_to(request.user).order_by('corp__corporation_name').select_related('corp')

    # ensure we can see the requested model
    if corpstats and corpstats not in available:
        raise PermissionDenied('You do not have permission to view the selected corporation statistics module.')

    # get default model if none requested
    if not corp_id and available.count() == 1:
        corpstats = available[0]
    elif not corp_id and available.count() > 1 and request.user.profile.main_character:
        # get their main corp if available
        try:
            corpstats = available.get(corp__corporation_id=request.user.profile.main_character.corporation_id)
        except CorpStats.DoesNotExist:
            pass

    context = {
        'available': available,
    }

    if corpstats:
        character_list = CorpMember.objects.filter(corpstats=corpstats)
        linked_chars = EveCharacter.objects.filter(
            character_id__in=character_list.values_list('character_id', flat=True))
        linked_chars = linked_chars | EveCharacter.objects.filter(
            character_ownership__user__profile__main_character__corporation_id=corpstats.corp.corporation_id)

        linked_chars = linked_chars.select_related('character_ownership',
                                                    'character_ownership__user__profile__main_character') \
            .prefetch_related('character_ownership__user__character_ownerships') \
            .prefetch_related('character_ownership__user__character_ownerships__character')

        members = []
        mains = {}

        temp_ids = []
        for char in linked_chars:
            try:
                main = char.character_ownership.user.profile.main_character
                if main is not None:
                    if main.corporation_id == corpstats.corp.corporation_id:
                        if main.character_id not in mains:
                            mains[main.character_id] = {'main':main, 'alts':[]}

                        mains[main.character_id]['alts'].append(char)

                    if char.corporation_id == corpstats.corp.corporation_id:
                        members.append(char)

                    temp_ids.append(char.character_id)

            except ObjectDoesNotExist:
                pass

        unregistered = character_list.exclude(character_id__in=temp_ids)

        members = members
        mains = mains
        total_mains = len(mains)
        unregistered = unregistered
        context.update({
            'corpstats': corpstats,
            'members': members,
            'mains': mains,
            'total_mains': total_mains,
            'unregistered': unregistered,
        })

    return render(request, 'corputils/corpstats.html', context=context)


@login_required
@user_passes_test(access_corpstats_test)
def corpstats_update(request, corp_id):
    corp = get_object_or_404(EveCorporationInfo, corporation_id=corp_id)
    corpstats = get_object_or_404(CorpStats, corp=corp)
    try:
        corpstats.update()
    except HTTPError as e:
        messages.error(request, str(e))
    if corpstats.pk:
        return redirect('corputils:view_corp', corp_id=corp.corporation_id)
    else:
        return redirect('corputils:view')


@login_required
@user_passes_test(access_corpstats_test)
def corpstats_search(request):
    results = []
    search_string = request.GET.get('search_string', None)
    if search_string:
        has_similar = CorpStats.objects.filter(members__character_name__icontains=search_string).visible_to(
            request.user).distinct()
        for corpstats in has_similar:
            similar = corpstats.members.filter(character_name__icontains=search_string)
            for s in similar:
                results.append((corpstats, s))
        results = sorted(results, key=lambda x: x[1].character_name)
        context = {
            'available': CorpStats.objects.visible_to(request.user),
            'results': results,
            'search_string': search_string,
        }
        return render(request, 'corputils/search.html', context=context)
    return redirect('corputils:view')



# Generated by Django 3.1.13 on 2021-10-12 20:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0015_factions'),
        ('authentication', '0017_remove_fleetup_permission'),
    ]

    operations = [
        migrations.AddField(
            model_name='state',
            name='member_factions',
            field=models.ManyToManyField(blank=True, help_text='Factions to whose members this state is available.', to='eveonline.EveFactionInfo'),
        ),
    ]



from django.contrib.auth.models import User
from django.test import TestCase

from allianceauth.eveonline.models import EveCorporationInfo
from allianceauth.tests.auth_utils import AuthUtils

from .models import Application, ApplicationForm, ApplicationQuestion, ApplicationChoice


class TestApplicationManagersPendingRequestsCountForUser(TestCase):
    def setUp(self) -> None:
        self.corporation_1 = EveCorporationInfo.objects.create(
            corporation_id=2001, corporation_name="Wayne Tech", member_count=42
        )
        self.corporation_2 = EveCorporationInfo.objects.create(
            corporation_id=2011, corporation_name="Lex Corp", member_count=666
        )
        question = ApplicationQuestion.objects.create(title="Dummy Question")
        ApplicationChoice.objects.create(question=question, choice_text="yes")
        ApplicationChoice.objects.create(question=question, choice_text="no")
        self.form_corporation_1 = ApplicationForm.objects.create(
            corp=self.corporation_1
        )
        self.form_corporation_1.questions.add(question)
        self.form_corporation_2 = ApplicationForm.objects.create(
            corp=self.corporation_2
        )
        self.form_corporation_2.questions.add(question)

        self.user_requestor = AuthUtils.create_member("Peter Parker")

        self.user_manager = AuthUtils.create_member("Bruce Wayne")
        AuthUtils.add_main_character_2(
            self.user_manager,
            self.user_manager.username,
            1001,
            self.corporation_1.corporation_id,
            self.corporation_1.corporation_name,
        )
        AuthUtils.add_permission_to_user_by_name(
            "auth.human_resources", self.user_manager
        )
        self.user_manager = User.objects.get(pk=self.user_manager.pk)

    def test_no_pending_application(self):
        # given manager of corporation 1 has permission
        # when no application is pending for corporation 1
        # return 0
        self.assertEqual(
            Application.objects.pending_requests_count_for_user(self.user_manager), 0
        )

    def test_single_pending_application(self):
        # given manager of corporation 1 has permission
        # when 1 application is pending for corporation 1
        # return 1
        Application.objects.create(
            form=self.form_corporation_1, user=self.user_requestor
        )
        self.assertEqual(
            Application.objects.pending_requests_count_for_user(self.user_manager), 1
        )

    def test_user_has_no_permission(self):
        # given user has no permission
        # when 1 application is pending
        # return None
        self.assertIsNone(
            Application.objects.pending_requests_count_for_user(self.user_requestor)
        )

    def test_two_pending_applications_for_different_corporations_normal_manager(self):
        # given manager of corporation 1 has permission
        # when 1 application is pending for corporation 1
        # and 1 application is pending for corporation 2
        # return 1
        Application.objects.create(
            form=self.form_corporation_1, user=self.user_requestor
        )
        Application.objects.create(
            form=self.form_corporation_2, user=self.user_requestor
        )
        self.assertEqual(
            Application.objects.pending_requests_count_for_user(self.user_manager), 1
        )

    def test_two_pending_applications_for_different_corporations_manager_is_super(self):
        # given manager of corporation 1 has permission
        # when 1 application is pending for corporation 1
        # and 1 application is pending for corporation 2
        # return 1
        Application.objects.create(
            form=self.form_corporation_1, user=self.user_requestor
        )
        Application.objects.create(
            form=self.form_corporation_2, user=self.user_requestor
        )
        superuser = User.objects.create_superuser(
            "Superman", "superman@example.com", "password"
        )
        self.assertEqual(
            Application.objects.pending_requests_count_for_user(superuser), 2
        )



# Generated by Django 2.2.12 on 2020-05-25 02:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0010_alliance_ticker'),
    ]

    operations = [
        migrations.AlterField(
            model_name='eveallianceinfo',
            name='alliance_id',
            field=models.PositiveIntegerField(unique=True),
        ),
        migrations.AlterField(
            model_name='eveallianceinfo',
            name='executor_corp_id',
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterField(
            model_name='evecharacter',
            name='alliance_id',
            field=models.PositiveIntegerField(blank=True, default=None, null=True),
        ),
        migrations.AlterField(
            model_name='evecharacter',
            name='character_id',
            field=models.PositiveIntegerField(unique=True),
        ),
        migrations.AlterField(
            model_name='evecharacter',
            name='corporation_id',
            field=models.PositiveIntegerField(),
        ),
        migrations.AlterField(
            model_name='evecorporationinfo',
            name='corporation_id',
            field=models.PositiveIntegerField(unique=True),
        ),
    ]



# Generated by Django 1.10.5 on 2017-03-27 03:29

from django.db import migrations
import sortedm2m.fields
from sortedm2m.operations import AlterSortedManyToManyField


class Migration(migrations.Migration):

    dependencies = [
        ('hrapplications', '0004_make_strings_more_stringy'),
    ]

    operations = [
        AlterSortedManyToManyField(
            model_name='applicationform',
            name='questions',
            field=sortedm2m.fields.SortedManyToManyField(help_text=None, to='hrapplications.ApplicationQuestion'),
        ),
    ]



import logging

from django.contrib.auth.models import User
from django.template.loader import render_to_string

from allianceauth import hooks
from allianceauth.services.hooks import ServicesHook

from .models import DiscordUser
from .urls import urlpatterns
from .utils import LoggerAddTag
from . import tasks, __title__


logger = LoggerAddTag(logging.getLogger(__name__), __title__)

# Default priority for single tasks like update group and sync nickname
SINGLE_TASK_PRIORITY = 3


class DiscordService(ServicesHook):
    """Service for managing a Discord server with Auth"""

    def __init__(self):
        ServicesHook.__init__(self)
        self.urlpatterns = urlpatterns
        self.name = 'discord'
        self.service_ctrl_template = 'services/discord/discord_service_ctrl.html'
        self.access_perm = 'discord.access_discord'
        self.name_format = '{character_name}'

    def delete_user(self, user: User, notify_user: bool = False) -> None:
        if self.user_has_account(user):
            logger.debug('Deleting user %s %s account', user, self.name)
            tasks.delete_user.apply_async(
                kwargs={'user_pk': user.pk, 'notify_user': notify_user},
                priority=SINGLE_TASK_PRIORITY
            )

    def render_services_ctrl(self, request):
        if self.user_has_account(request.user):
            user_has_account = True
            username = request.user.discord.username
            discriminator = request.user.discord.discriminator
            if username and discriminator:
                discord_username = f'{username}#{discriminator}'
            else:
                discord_username = ''
        else:
            discord_username = ''
            user_has_account = False

        return render_to_string(
            self.service_ctrl_template,
            {
                'server_name': DiscordUser.objects.server_name(),
                'user_has_account': user_has_account,
                'discord_username': discord_username
            },
            request=request
        )

    def service_active_for_user(self, user):
        has_perms = user.has_perm(self.access_perm)
        logger.debug("User %s has service permission: %s", user, has_perms)
        return has_perms

    def sync_nickname(self, user):
        logger.debug('Syncing %s nickname for user %s', self.name, user)
        if self.user_has_account(user):
            tasks.update_nickname.apply_async(
                kwargs={
                    'user_pk': user.pk,
                    # since the new nickname is not yet in the DB we need to
                    # provide it manually to the task
                    'nickname': DiscordUser.objects.user_formatted_nick(user)
                },
                priority=SINGLE_TASK_PRIORITY
            )

    def sync_nicknames_bulk(self, users: list):
        """Sync nickname for a list of users in bulk.
        Preferred over sync_nickname(), because it will not break the rate limit
        """
        logger.debug(
            'Syncing %s nicknames in bulk for %d users', self.name, len(users)
        )
        user_pks = [user.pk for user in users]
        tasks.update_nicknames_bulk.delay(user_pks)

    def update_all_groups(self):
        logger.debug('Update all %s groups called', self.name)
        tasks.update_all_groups.delay()

    def update_groups(self, user):
        logger.debug('Processing %s groups for %s', self.name, user)
        if self.user_has_account(user):
            tasks.update_groups.apply_async(
                kwargs={
                    'user_pk': user.pk,
                    # since state changes may not yet be in the DB we need to
                    # provide the new state name manually to the task
                    'state_name': user.profile.state.name
                },
                priority=SINGLE_TASK_PRIORITY
            )

    def update_groups_bulk(self, users: list):
        """Updates groups for a list of users in bulk.
        Preferred over update_groups(), because it will not break the rate limit
        """
        logger.debug(
            'Processing %s groups in bulk for %d users', self.name, len(users)
        )
        user_pks = [user.pk for user in users]
        tasks.update_groups_bulk.delay(user_pks)

    @staticmethod
    def user_has_account(user: User) -> bool:
        result = DiscordUser.objects.user_has_account(user)
        if result:
            logger.debug('User %s has a Discord account', user)
        else:
            logger.debug('User %s does not have a Discord account', user)
        return result

    def validate_user(self, user):
        logger.debug('Validating user %s %s account', user, self.name)
        if self.user_has_account(user) and not self.service_active_for_user(user):
            self.delete_user(user, notify_user=True)


@hooks.register('services_hook')
def register_service():
    return DiscordService()



from hashlib import md5
import json
import logging
from time import sleep
from urllib.parse import urljoin
from uuid import uuid1

from redis import Redis
import requests

from django.core.cache import caches

from allianceauth import __title__ as AUTH_TITLE, __url__, __version__

from .. import __title__
from .app_settings import (
    DISCORD_API_BASE_URL,
    DISCORD_API_TIMEOUT_CONNECT,
    DISCORD_API_TIMEOUT_READ,
    DISCORD_DISABLE_ROLE_CREATION,
    DISCORD_GUILD_NAME_CACHE_MAX_AGE,
    DISCORD_OAUTH_BASE_URL,
    DISCORD_OAUTH_TOKEN_URL,
    DISCORD_ROLES_CACHE_MAX_AGE,
)
from .exceptions import DiscordRateLimitExhausted, DiscordTooManyRequestsError
from .helpers import DiscordRoles
from ..utils import LoggerAddTag


logger = LoggerAddTag(logging.getLogger(__name__), __title__)

# max requests that can be executed until reset
RATE_LIMIT_MAX_REQUESTS = 5

# Time until remaining requests are reset
RATE_LIMIT_RESETS_AFTER = 5000

# Delay used for API backoff in case no info returned from API on 429s
DEFAULT_BACKOFF_DELAY = 5000

# additional duration to compensate for potential clock discrepancies
# with the Discord server
DURATION_CONTINGENCY = 500

# Client will do a blocking wait rather than throwing a backoff exception if the
# time until next reset is below this threshold
WAIT_THRESHOLD = 250

# Minimum wait duration when doing a blocking wait
MINIMUM_BLOCKING_WAIT = 50

# If the rate limit resets soon we will wait it out and then retry to
# either get a remaining request from our cached counter
# or again wait out a short reset time and retry again.
# This could happen several times within a high concurrency situation,
# but must fail after x tries to avoid an infinite loop
RATE_LIMIT_RETRIES = 1000


class DiscordClient:
    """This class provides a web client for interacting with the Discord API

    The client has rate limiting that supports concurrency.
    This means it is able to ensure the API rate limit is not violated,
    even when used concurrently, e.g. with multiple parallel celery tasks.

    In addition the client support proper API backoff.

    Synchronization of rate limit infos accross multiple processes
    is implemented with Redis and thus requires Redis as Django cache backend.

    All durations are in milliseconds.
    """
    OAUTH_BASE_URL = DISCORD_OAUTH_BASE_URL
    OAUTH_TOKEN_URL = DISCORD_OAUTH_TOKEN_URL

    _KEY_GLOBAL_BACKOFF_UNTIL = 'DISCORD_GLOBAL_BACKOFF_UNTIL'
    _KEY_GLOBAL_RATE_LIMIT_REMAINING = 'DISCORD_GLOBAL_RATE_LIMIT_REMAINING'
    _KEYPREFIX_GUILD_NAME = 'DISCORD_GUILD_NAME'
    _KEYPREFIX_GUILD_ROLES = 'DISCORD_GUILD_ROLES'
    _KEYPREFIX_ROLE_NAME = 'DISCORD_ROLE_NAME'
    _NICK_MAX_CHARS = 32

    _HTTP_STATUS_CODE_NOT_FOUND = 404
    _HTTP_STATUS_CODE_RATE_LIMITED = 429
    _DISCORD_STATUS_CODE_UNKNOWN_MEMBER = 10007

    def __init__(
        self,
        access_token: str,
        redis: Redis = None,
        is_rate_limited: bool = True
    ) -> None:
        """
        Params:
        - access_token: Discord access token used to authenticate all calls to the API
        - redis: Redis instance to be used.
        - is_rate_limited: Set to False to run of rate limiting (use with care)
        If not specified will try to use the Redis instance
        from the default Django cache backend.
        """
        self._access_token = str(access_token)
        self._is_rate_limited = bool(is_rate_limited)
        if not redis:
            default_cache = caches['default']
            self._redis = default_cache.get_master_client()
            if not isinstance(self._redis, Redis):
                raise RuntimeError(
                    'This class requires a Redis client, but none was provided '
                    'and the default Django cache backend is not Redis either.'
                )
        else:
            self._redis = redis

        lua_1 = """
            if redis.call("exists", KEYS[1]) == 0 then
                redis.call("set", KEYS[1], ARGV[1], 'px', ARGV[2])
            end
            return redis.call("decr", KEYS[1])
        """
        self.__redis_script_decr_or_set = self._redis.register_script(lua_1)

        lua_2 = """
            local current_px = tonumber(redis.call("pttl", KEYS[1]))
            if current_px < tonumber(ARGV[2]) then
                return redis.call("set", KEYS[1], ARGV[1], 'px', ARGV[2])
            else
                return nil
            end
        """
        self.__redis_script_set_longer = self._redis.register_script(lua_2)

    @property
    def access_token(self):
        return self._access_token

    @property
    def is_rate_limited(self):
        return self._is_rate_limited

    def __repr__(self):
        return f'{type(self).__name__}(access_token=...{self.access_token[-5:]})'

    def _redis_decr_or_set(self, name: str, value: str, px: int) -> bool:
        """decreases the key value if it exists and returns the result
        else sets the key

        Implemented as Lua script to ensure atomicity.
        """
        return self.__redis_script_decr_or_set(
            keys=[str(name)], args=[str(value), int(px)]
        )

    def _redis_set_if_longer(self, name: str, value: str, px: int) -> bool:
        """like set, but only goes through if either key doesn't exist
        or px would be extended.

        Implemented as Lua script to ensure atomicity.
        """
        return self.__redis_script_set_longer(
            keys=[str(name)], args=[str(value), int(px)]
        )

    # users

    def current_user(self) -> dict:
        """returns the user belonging to the current access_token"""
        authorization = f'Bearer {self.access_token}'
        r = self._api_request(
            method='get', route='users/@me', authorization=authorization
        )
        return r.json()

    # guild

    def guild_infos(self, guild_id: int) -> dict:
        """Returns all basic infos about this guild"""
        route = f"guilds/{guild_id}"
        r = self._api_request(method='get', route=route)
        return r.json()

    def guild_name(self, guild_id: int, use_cache: bool = True) -> str:
        """returns the name of this guild (cached)
        or an empty string if something went wrong

        Params:
        - guild_id: ID of current guild
        - use_cache: When set to False will force an API call to get the server name
        """
        key_name = self._guild_name_cache_key(guild_id)
        if use_cache:
            guild_name = self._redis_decode(self._redis.get(key_name))
        else:
            guild_name = None
        if not guild_name:
            guild_infos = self.guild_infos(guild_id)
            if 'name' in guild_infos:
                guild_name = guild_infos['name']
                self._redis.set(
                    name=key_name,
                    value=guild_name,
                    ex=DISCORD_GUILD_NAME_CACHE_MAX_AGE
                )
            else:
                guild_name = ''

        return guild_name

    @classmethod
    def _guild_name_cache_key(cls, guild_id: int) -> str:
        """Returns key for accessing role given by name in the role cache"""
        gen_key = DiscordClient._generate_hash(f'{guild_id}')
        return f'{cls._KEYPREFIX_GUILD_NAME}__{gen_key}'

    # guild roles

    def guild_roles(self, guild_id: int, use_cache: bool = True) -> list:
        """Returns the list of all roles for this guild

        If use_cache is set to False it will always hit the API to retrieve
        fresh data and update the cache
        """
        cache_key = self._guild_roles_cache_key(guild_id)
        if use_cache:
            roles_raw = self._redis.get(name=cache_key)
            if roles_raw:
                logger.debug('Returning roles for guild %s from cache', guild_id)
                return json.loads(self._redis_decode(roles_raw))
            else:
                logger.debug('No roles for guild %s in cache', guild_id)

        route = f"guilds/{guild_id}/roles"
        r = self._api_request(method='get', route=route)
        roles = r.json()
        if roles and isinstance(roles, list):
            self._redis.set(
                name=cache_key,
                value=json.dumps(roles),
                ex=DISCORD_ROLES_CACHE_MAX_AGE
            )
        return roles

    def create_guild_role(self, guild_id: int, role_name: str, **kwargs) -> dict:
        """Create a new guild role with the given name.
        See official documentation for additional optional parameters.

        Note that Discord allows the creation of multiple roles with the same name,
        so to avoid duplicates it's important to check existing roles
        before creating new one

        returns a new role dict on success
        """
        route = f"guilds/{guild_id}/roles"
        data = {'name': DiscordRoles.sanitize_role_name(role_name)}
        data.update(kwargs)
        r = self._api_request(method='post', route=route, data=data)
        role = r.json()
        if role:
            self._invalidate_guild_roles_cache(guild_id)
        return role

    def delete_guild_role(self, guild_id: int, role_id: int) -> bool:
        """Deletes a guild role"""
        route = f"guilds/{guild_id}/roles/{role_id}"
        r = self._api_request(method='delete', route=route)
        if r.status_code == 204:
            self._invalidate_guild_roles_cache(guild_id)
            return True
        else:
            return False

    def _invalidate_guild_roles_cache(self, guild_id: int) -> None:
        cache_key = self._guild_roles_cache_key(guild_id)
        self._redis.delete(cache_key)
        logger.debug('Guild roles cache invalidated')

    @classmethod
    def _guild_roles_cache_key(cls, guild_id: int) -> str:
        """Returns key for accessing cached roles for a guild"""
        gen_key = cls._generate_hash(f'{guild_id}')
        return f'{cls._KEYPREFIX_GUILD_ROLES}__{gen_key}'

    def match_role_from_name(self, guild_id: int, role_name: str) -> dict:
        """returns Discord role matching the given name or an empty dict"""
        guild_roles = DiscordRoles(self.guild_roles(guild_id))
        return guild_roles.role_by_name(role_name)

    def match_or_create_roles_from_names(self, guild_id: int, role_names: list) -> list:
        """returns Discord roles matching the given names

        Returns as list of tuple of role and created flag

        Will try to match with existing roles names
        Non-existing roles will be created, then created flag will be True

        Params:
        - guild_id: ID of guild
        - role_names: list of name strings each defining a role
        """
        roles = list()
        guild_roles = DiscordRoles(self.guild_roles(guild_id))
        role_names_cleaned = {
            DiscordRoles.sanitize_role_name(name) for name in role_names
        }
        for role_name in role_names_cleaned:
            role, created = self.match_or_create_role_from_name(
                guild_id=guild_id,
                role_name=DiscordRoles.sanitize_role_name(role_name),
                guild_roles=guild_roles
            )
            if role:
                roles.append((role, created))
            if created:
                guild_roles = guild_roles.union(DiscordRoles([role]))
        return roles

    def match_or_create_role_from_name(
        self, guild_id: int, role_name: str, guild_roles: DiscordRoles = None
    ) -> tuple:
        """returns Discord role matching the given name

        Returns as tuple of role and created flag

        Will try to match with existing roles names
        Non-existing roles will be created, then created flag will be True

        Params:
        - guild_id: ID of guild
        - role_name: strings defining name of a role
        - guild_roles: All known guild roles as DiscordRoles object.
        Helps to void redundant lookups of guild roles
        when this method is used multiple times.
        """
        if not isinstance(role_name, str):
            raise TypeError('role_name must be of type string')

        created = False
        if guild_roles is None:
            guild_roles = DiscordRoles(self.guild_roles(guild_id))
        role = guild_roles.role_by_name(role_name)
        if not role:
            if not DISCORD_DISABLE_ROLE_CREATION:
                logger.debug('Need to create missing role: %s', role_name)
                role = self.create_guild_role(guild_id, role_name)
                created = True
            else:
                role = None

        return role, created

    # guild members

    def add_guild_member(
        self,
        guild_id: int,
        user_id: int,
        access_token: str,
        role_ids: list = None,
        nick: str = None
    ) -> bool:
        """Adds a user to the guilds.

        Returns:
        - True when a new user was added
        - None if the user already existed
        - False when something went wrong or raises exception
        """
        route = f"guilds/{guild_id}/members/{user_id}"
        data = {
            'access_token': str(access_token)
        }
        if role_ids:
            data['roles'] = self._sanitize_role_ids(role_ids)

        if nick:
            data['nick'] = str(nick)[:self._NICK_MAX_CHARS]

        r = self._api_request(method='put', route=route, data=data)
        r.raise_for_status()
        if r.status_code == 201:
            return True
        elif r.status_code == 204:
            return None
        else:
            return False

    def guild_member(self, guild_id: int, user_id: int) -> dict:
        """returns the user info for a guild member

        or None if the user is not a member of the guild
        """
        route = f'guilds/{guild_id}/members/{user_id}'
        r = self._api_request(method='get', route=route, raise_for_status=False)
        if self._is_member_unknown_error(r):
            logger.warning("Discord user ID %s could not be found on server.", user_id)
            return None
        else:
            r.raise_for_status()
            return r.json()

    def modify_guild_member(
        self, guild_id: int, user_id: int, role_ids: list = None, nick: str = None
    ) -> bool:
        """Modify attributes of a guild member.

        Returns
        - True when successful
        - None if user is not a member of this guild
        - False otherwise
        """
        if not role_ids and not nick:
            raise ValueError('Must specify role_ids or nick')

        if role_ids and not isinstance(role_ids, list):
            raise TypeError('role_ids must be a list type')

        data = dict()
        if role_ids:
            data['roles'] = self._sanitize_role_ids(role_ids)

        if nick:
            data['nick'] = self._sanitize_nick(nick)

        route = f"guilds/{guild_id}/members/{user_id}"
        r = self._api_request(
            method='patch', route=route, data=data, raise_for_status=False
        )
        if self._is_member_unknown_error(r):
            logger.warning('User ID %s is not a member of this guild', user_id)
            return None
        else:
            r.raise_for_status()

        if r.status_code == 204:
            return True
        else:
            return False

    def remove_guild_member(self, guild_id: int, user_id: int) -> bool:
        """Remove a member from a guild

        Returns:
        - True when successful
        - None if member does not exist
        - False otherwise
        """
        route = f"guilds/{guild_id}/members/{user_id}"
        r = self._api_request(
            method='delete', route=route, raise_for_status=False
        )
        if self._is_member_unknown_error(r):
            logger.warning('User ID %s is not a member of this guild', user_id)
            return None
        else:
            r.raise_for_status()

        if r.status_code == 204:
            return True
        else:
            return False

    # Guild member roles

    def add_guild_member_role(
        self, guild_id: int, user_id: int, role_id: int
    ) -> bool:
        """Adds a role to a guild member

        Returns:
        - True when successful
        - None if member does not exist
        - False otherwise
        """
        route = f"guilds/{guild_id}/members/{user_id}/roles/{role_id}"
        r = self._api_request(method='put', route=route, raise_for_status=False)
        if self._is_member_unknown_error(r):
            logger.warning('User ID %s is not a member of this guild', user_id)
            return None
        else:
            r.raise_for_status()

        if r.status_code == 204:
            return True
        else:
            return False

    def remove_guild_member_role(
        self, guild_id: int, user_id: int, role_id: int
    ) -> bool:
        """Removes a role to a guild member

        Returns:
        - True when successful
        - None if member does not exist
        - False otherwise
        """
        route = f"guilds/{guild_id}/members/{user_id}/roles/{role_id}"
        r = self._api_request(method='delete', route=route, raise_for_status=False)
        if self._is_member_unknown_error(r):
            logger.warning('User ID %s is not a member of this guild', user_id)
            return None
        else:
            r.raise_for_status()

        if r.status_code == 204:
            return True
        else:
            return False

    @classmethod
    def _is_member_unknown_error(cls, r: requests.Response) -> bool:
        try:
            result = (
                r.status_code == cls._HTTP_STATUS_CODE_NOT_FOUND
                and r.json()['code'] == cls._DISCORD_STATUS_CODE_UNKNOWN_MEMBER
            )
        except (ValueError, KeyError):
            result = False

        return result

    # Internal methods

    def _api_request(
        self,
        method: str,
        route: str,
        data: dict = None,
        authorization: str = None,
        raise_for_status: bool = True
    ) -> requests.Response:
        """Core method for performing all API calls"""
        uid = uuid1().hex

        if not hasattr(requests, method):
            raise ValueError('Invalid method: %s' % method)

        if not authorization:
            authorization = f'Bot {self.access_token}'

        self._handle_ongoing_api_backoff(uid)
        if self.is_rate_limited:
            self._ensure_rate_limed_not_exhausted(uid)
        headers = {
            'User-Agent': f'{AUTH_TITLE} ({__url__}, {__version__})',
            'accept': 'application/json',
            'X-RateLimit-Precision': 'millisecond',
            'authorization': str(authorization)
        }
        if data:
            headers['content-type'] = 'application/json'

        url = urljoin(DISCORD_API_BASE_URL, route)
        args = {
            'url': url,
            'headers': headers,
            'timeout': (DISCORD_API_TIMEOUT_CONNECT, DISCORD_API_TIMEOUT_READ)
        }
        if data:
            args['json'] = data

        logger.info('%s: sending %s request to url \'%s\'', uid, method.upper(), url)
        logger.debug('%s: request headers: %s', uid, headers)
        r = getattr(requests, method)(**args)
        logger.debug(
            '%s: returned status code %d with headers: %s',
            uid,
            r.status_code,
            r.headers
        )
        logger.debug('%s: response:\n%s', uid, r.text)
        if not r.ok:
            logger.warning(
                '%s: Discord API returned error code %d and this response: %s',
                uid,
                r.status_code,
                r.text
            )

        if r.status_code == self._HTTP_STATUS_CODE_RATE_LIMITED:
            self._handle_new_api_backoff(r, uid)

        self._report_rate_limit_from_api(r, uid)

        if raise_for_status:
            r.raise_for_status()

        return r

    def _handle_ongoing_api_backoff(self, uid: str) -> None:
        """checks if api is currently on backoff
        if on backoff: will do a blocking wait if it expires soon,
        else raises exception
        """
        global_backoff_duration = self._redis.pttl(self._KEY_GLOBAL_BACKOFF_UNTIL)
        if global_backoff_duration > 0:
            if global_backoff_duration < WAIT_THRESHOLD:
                logger.info(
                    '%s: Global API backoff still ongoing for %s ms. Waiting.',
                    uid,
                    global_backoff_duration
                )
                sleep(global_backoff_duration / 1000)
            else:
                logger.info(
                    '%s: Global API backoff still ongoing for %s ms. Re-raising.',
                    uid,
                    global_backoff_duration
                )
                raise DiscordTooManyRequestsError(retry_after=global_backoff_duration)

    def _ensure_rate_limed_not_exhausted(self, uid: str) -> int:
        """ensures that the rate limit is not exhausted
        if exhausted: will do a blocking wait if rate limit resets soon,
        else raises exception

        returns requests remaining on success
        """
        for _ in range(RATE_LIMIT_RETRIES):
            requests_remaining = self._redis_decr_or_set(
                name=self._KEY_GLOBAL_RATE_LIMIT_REMAINING,
                value=RATE_LIMIT_MAX_REQUESTS,
                px=RATE_LIMIT_RESETS_AFTER + DURATION_CONTINGENCY
            )
            resets_in = max(
                MINIMUM_BLOCKING_WAIT,
                self._redis.pttl(self._KEY_GLOBAL_RATE_LIMIT_REMAINING)
            )
            if requests_remaining >= 0:
                logger.debug(
                    '%s: Got one of %d remaining requests until reset in %s ms',
                    uid,
                    requests_remaining + 1,
                    resets_in
                )
                return requests_remaining

            elif resets_in < WAIT_THRESHOLD:
                sleep(resets_in / 1000)
                logger.debug(
                    '%s: No requests remaining until reset in %d ms. '
                    'Waiting for reset.',
                    uid,
                    resets_in
                )
                continue

            else:
                logger.debug(
                    '%s: No requests remaining until reset in %d ms. '
                    'Raising exception.',
                    uid,
                    resets_in
                )
                raise DiscordRateLimitExhausted(resets_in)

        raise RuntimeError('Failed to handle rate limit after after too tries.')

    def _handle_new_api_backoff(self, r: requests.Response, uid: str) -> None:
        """raises exception for new API backoff error"""
        response = r.json()
        if 'retry_after' in response:
            try:
                retry_after = \
                    int(response['retry_after']) + DURATION_CONTINGENCY
            except ValueError:
                retry_after = DEFAULT_BACKOFF_DELAY
        else:
            retry_after = DEFAULT_BACKOFF_DELAY
        self._redis_set_if_longer(
            name=self._KEY_GLOBAL_BACKOFF_UNTIL,
            value='GLOBAL_API_BACKOFF',
            px=retry_after
        )
        logger.warning(
            "%s: Rate limit violated. Need to back off for at least %d ms",
            uid,
            retry_after
        )
        raise DiscordTooManyRequestsError(retry_after=retry_after)

    def _report_rate_limit_from_api(self, r, uid):
        """Tries to log the current rate limit reported from API"""
        if (
            logger.getEffectiveLevel() <= logging.DEBUG
            and 'x-ratelimit-limit' in r.headers
            and 'x-ratelimit-remaining' in r.headers
            and 'x-ratelimit-reset-after' in r.headers
        ):
            try:
                limit = int(r.headers['x-ratelimit-limit'])
                remaining = int(r.headers['x-ratelimit-remaining'])
                reset_after = float(r.headers['x-ratelimit-reset-after']) * 1000
                if remaining + 1 == limit:
                    logger.debug(
                        '%s: Rate limit reported from API: %d requests per %s ms',
                        uid,
                        limit,
                        reset_after
                    )
            except ValueError:
                pass

    @staticmethod
    def _redis_decode(value: str) -> str:
        """Decodes a string from Redis and passes through None and Booleans"""
        if value is not None and not isinstance(value, bool):
            return value.decode('utf-8')
        else:
            return value

    @staticmethod
    def _generate_hash(key: str) -> str:
        return md5(key.encode('utf-8')).hexdigest()

    @staticmethod
    def _sanitize_role_ids(role_ids: list) -> list:
        """make sure its a list of integers"""
        return [int(role_id) for role_id in list(role_ids)]

    @classmethod
    def _sanitize_nick(cls, nick: str) -> str:
        """shortens too long strings if necessary"""
        return str(nick)[:cls._NICK_MAX_CHARS]



from unittest.mock import patch
from urllib.parse import parse_qs

import requests_mock

from django.test import TestCase, override_settings

from allianceauth.analytics.tasks import ANALYTICS_URL
from allianceauth.eveonline.tasks import update_character
from allianceauth.tests.auth_utils import AuthUtils


@override_settings(CELERY_ALWAYS_EAGER=True)
@requests_mock.mock()
class TestAnalyticsForViews(TestCase):
    @override_settings(ANALYTICS_DISABLED=False)
    def test_should_run_analytics(self, requests_mocker):
        # given
        requests_mocker.post(ANALYTICS_URL)
        user = AuthUtils.create_user("Bruce Wayne")
        self.client.force_login(user)
        # when
        response = self.client.get("/dashboard/")
        # then
        self.assertEqual(response.status_code, 200)
        self.assertTrue(requests_mocker.called)

    @override_settings(ANALYTICS_DISABLED=True)
    def test_should_not_run_analytics(self, requests_mocker):
        # given
        requests_mocker.post(ANALYTICS_URL)
        user = AuthUtils.create_user("Bruce Wayne")
        self.client.force_login(user)
        # when
        response = self.client.get("/dashboard/")
        # then
        self.assertEqual(response.status_code, 200)
        self.assertFalse(requests_mocker.called)


@override_settings(CELERY_ALWAYS_EAGER=True)
@requests_mock.mock()
class TestAnalyticsForTasks(TestCase):
    @override_settings(ANALYTICS_DISABLED=False)
    @patch("allianceauth.eveonline.models.EveCharacter.objects.update_character")
    def test_should_run_analytics_for_successful_task(
        self, requests_mocker, mock_update_character
    ):
        # given
        requests_mocker.post(ANALYTICS_URL)
        user = AuthUtils.create_user("Bruce Wayne")
        character = AuthUtils.add_main_character_2(user, "Bruce Wayne", 1001)
        # when
        update_character.delay(character.character_id)
        # then
        self.assertTrue(mock_update_character.called)
        self.assertTrue(requests_mocker.called)
        payload = parse_qs(requests_mocker.last_request.text)
        self.assertListEqual(payload["el"], ["Success"])

    @override_settings(ANALYTICS_DISABLED=True)
    @patch("allianceauth.eveonline.models.EveCharacter.objects.update_character")
    def test_should_not_run_analytics_for_successful_task(
        self, requests_mocker, mock_update_character
    ):
        # given
        requests_mocker.post(ANALYTICS_URL)
        user = AuthUtils.create_user("Bruce Wayne")
        character = AuthUtils.add_main_character_2(user, "Bruce Wayne", 1001)
        # when
        update_character.delay(character.character_id)
        # then
        self.assertTrue(mock_update_character.called)
        self.assertFalse(requests_mocker.called)

    @override_settings(ANALYTICS_DISABLED=False)
    @patch("allianceauth.eveonline.models.EveCharacter.objects.update_character")
    def test_should_run_analytics_for_failed_task(
        self, requests_mocker, mock_update_character
    ):
        # given
        requests_mocker.post(ANALYTICS_URL)
        mock_update_character.side_effect = RuntimeError
        user = AuthUtils.create_user("Bruce Wayne")
        character = AuthUtils.add_main_character_2(user, "Bruce Wayne", 1001)
        # when
        update_character.delay(character.character_id)
        # then
        self.assertTrue(mock_update_character.called)
        self.assertTrue(requests_mocker.called)
        payload = parse_qs(requests_mocker.last_request.text)
        self.assertNotEqual(payload["el"], ["Success"])

    @override_settings(ANALYTICS_DISABLED=True)
    @patch("allianceauth.eveonline.models.EveCharacter.objects.update_character")
    def test_should_not_run_analytics_for_failed_task(
        self, requests_mocker, mock_update_character
    ):
        # given
        requests_mocker.post(ANALYTICS_URL)
        mock_update_character.side_effect = RuntimeError
        user = AuthUtils.create_user("Bruce Wayne")
        character = AuthUtils.add_main_character_2(user, "Bruce Wayne", 1001)
        # when
        update_character.delay(character.character_id)
        # then
        self.assertTrue(mock_update_character.called)
        self.assertFalse(requests_mocker.called)



from django.conf.urls import url
from django.contrib.auth.decorators import login_required
from django.views.generic.base import TemplateView

from . import views

app_name = 'authentication'

urlpatterns = [
    url(r'^$', views.index, name='index'),
    url(
        r'^account/login/$',
        TemplateView.as_view(template_name='public/login.html'),
        name='login'
    ),
    url(
        r'^account/characters/main/$',
        views.main_character_change,
        name='change_main_character'
    ),
    url(
        r'^account/characters/add/$',
        views.add_character,
        name='add_character'
    ),
    url(r'^dashboard/$', views.dashboard, name='dashboard'),
]



import math


class DiscordClientException(Exception):
    """Base Exception for the Discord client"""


class DiscordApiBackoff(DiscordClientException):
    """Exception signaling we need to backoff from sending requests to the API for now
    """

    def __init__(self, retry_after: int):
        """
        :param retry_after: int time to retry after in milliseconds
        """
        super().__init__()
        self.retry_after = int(retry_after)

    @property
    def retry_after_seconds(self):
        return math.ceil(self.retry_after / 1000)


class DiscordRateLimitExhausted(DiscordApiBackoff):
    """Exception signaling that the total number of requests allowed under the
    current rate limit have been exhausted and weed to wait until next reset.
    """


class DiscordTooManyRequestsError(DiscordApiBackoff):
    """API has responded with a 429 Too Many Requests Error.
    Need to backoff for now.
    """



# Generated by Django 1.10.5 on 2017-03-22 23:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('srp', '0002_srpuserrequest_srp_status_choices'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='srpfleetmain',
            options={'permissions': (('access_srp', 'Can access SRP module'),)},
        ),
        migrations.AlterField(
            model_name='srpfleetmain',
            name='fleet_doctrine',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='srpfleetmain',
            name='fleet_name',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='srpfleetmain',
            name='fleet_srp_aar_link',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='srpfleetmain',
            name='fleet_srp_code',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='srpfleetmain',
            name='fleet_srp_status',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='srpuserrequest',
            name='additional_info',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='srpuserrequest',
            name='after_action_report_link',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='srpuserrequest',
            name='killboard_link',
            field=models.CharField(default='', max_length=254),
        ),
        migrations.AlterField(
            model_name='srpuserrequest',
            name='srp_ship_name',
            field=models.CharField(default='', max_length=254),
        ),
    ]



import esi.urls

from django.conf.urls import include, url
from django.contrib import admin
from django.views.generic.base import TemplateView

import allianceauth.authentication.views
import allianceauth.authentication.urls
import allianceauth.notifications.urls
import allianceauth.groupmanagement.urls
import allianceauth.services.urls
from allianceauth.authentication.decorators import main_character_required, decorate_url_patterns
from allianceauth import NAME
from allianceauth import views
from allianceauth.authentication import hmac_urls
from allianceauth.hooks import get_hooks

admin.site.site_header = NAME


# Functional/Untranslated URL's
urlpatterns = [
    # Locale
    url(r'^i18n/', include('django.conf.urls.i18n')),

    # Authentication
    url(r'', include(allianceauth.authentication.urls)),
    url(r'^account/login/$', TemplateView.as_view(template_name='public/login.html'), name='auth_login_user'),
    url(r'^account/', include(hmac_urls)),

    # Admin urls
    url(r'^admin/', admin.site.urls),

    # SSO
    url(r'^sso/', include((esi.urls, 'esi'), namespace='esi')),
    url(r'^sso/login$', allianceauth.authentication.views.sso_login, name='auth_sso_login'),

    # Notifications
    url(r'', include(allianceauth.notifications.urls)),

    # Groups
    url(r'', include(allianceauth.groupmanagement.urls)),

    # Services
    url(r'', decorate_url_patterns(allianceauth.services.urls.urlpatterns, main_character_required)),

    # Night mode
    url(r'^night/', views.NightModeRedirectView.as_view(), name='nightmode')
]


# Append app urls
app_urls = get_hooks('url_hook')
for app in app_urls:
    urlpatterns += [url(r'', decorate_url_patterns([app().include_pattern], main_character_required))]



from django.utils.translation import ugettext_lazy as _

from allianceauth.services.hooks import MenuItemHook, UrlHook
from allianceauth import hooks

from . import urls
from .managers import GroupManager


class GroupManagementMenuItem(MenuItemHook):
    """ This class ensures only authorized users will see the menu entry """

    def __init__(self):
        # setup menu entry for sidebar
        MenuItemHook.__init__(
            self,
            text=_("Group Management"),
            classes="fas fa-users-cog fa-fw",
            url_name="groupmanagement:management",
            order=50,
            navactive=[
                "groupmanagement:management",  # group requests view
                "groupmanagement:membership",  # group membership view
                "groupmanagement:audit_log",  # group audit log view
            ],
        )

    def render(self, request):
        if GroupManager.can_manage_groups(request.user):
            app_count = GroupManager.pending_requests_count_for_user(request.user)
            self.count = app_count if app_count and app_count > 0 else None
            return MenuItemHook.render(self, request)
        return ""


@hooks.register("menu_item_hook")
def register_menu():
    return GroupManagementMenuItem()


@hooks.register("url_hook")
def register_urls():
    return UrlHook(urls, "group", r"^groups/")



from unittest import mock

from django.test import TestCase, RequestFactory
from django import urls
from django.contrib.auth.models import User, Group, Permission
from django.core.exceptions import ObjectDoesNotExist
from django.db.models import signals
from django.contrib.admin import AdminSite

from allianceauth.tests.auth_utils import AuthUtils
from .auth_hooks import Teamspeak3Service
from .models import Teamspeak3User, AuthTS, TSgroup, StateGroup
from .tasks import Teamspeak3Tasks
from .signals import m2m_changed_authts_group, post_save_authts, post_delete_authts
from .admin import AuthTSgroupAdmin

from .manager import Teamspeak3Manager
from .util.ts3 import TeamspeakError
from allianceauth.groupmanagement.models import ReservedGroupName

MODULE_PATH = 'allianceauth.services.modules.teamspeak3'
DEFAULT_AUTH_GROUP = 'Member'


def add_permissions():
    permission = Permission.objects.get(codename='access_teamspeak3')
    members = Group.objects.get_or_create(name=DEFAULT_AUTH_GROUP)[0]
    AuthUtils.add_permissions_to_groups([permission], [members])


class Teamspeak3HooksTestCase(TestCase):
    def setUp(self):
        # Inert signals before setup begins
        with mock.patch(MODULE_PATH + '.signals.trigger_all_ts_update') as trigger_all_ts_update:
            self.member = 'member_user'
            member = AuthUtils.create_member(self.member)
            Teamspeak3User.objects.create(user=member, uid=self.member, perm_key='123ABC')
            self.none_user = 'none_user'
            AuthUtils.create_user(self.none_user)
            state = member.profile.state
            ts_member_group = TSgroup.objects.create(ts_group_id=1, ts_group_name='Member')
            ts_state_group = TSgroup.objects.create(ts_group_id=2, ts_group_name='State')
            m2m_member_group = AuthTS.objects.create(auth_group=member.groups.all()[0])
            m2m_member_group.ts_group.add(ts_member_group)
            m2m_member_group.save()
            StateGroup.objects.create(state=state, ts_group=ts_state_group)
            self.service = Teamspeak3Service
            add_permissions()

    def test_has_account(self):
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)
        self.assertTrue(Teamspeak3Tasks.has_account(member))
        self.assertFalse(Teamspeak3Tasks.has_account(none_user))

    def test_service_enabled(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        none_user = User.objects.get(username=self.none_user)

        self.assertTrue(service.service_active_for_user(member))
        self.assertFalse(service.service_active_for_user(none_user))

    @mock.patch(MODULE_PATH + '.tasks.Teamspeak3Manager')
    def test_update_all_groups(self, manager):
        instance = manager.return_value.__enter__.return_value
        service = self.service()
        service.update_all_groups()
        # Check user has groups updated
        self.assertTrue(instance.update_groups.called)
        self.assertEqual(instance.update_groups.call_count, 1)

    def test_update_groups(self):
        # Check member has Member group updated
        with mock.patch(MODULE_PATH + '.tasks.Teamspeak3Manager') as manager:
            instance = manager.return_value.__enter__.return_value
            service = self.service()
            member = User.objects.get(username=self.member)
            service.update_groups(member)
            self.assertTrue(instance.update_groups.called)
            args, kwargs = instance.update_groups.call_args
            # update_groups(user.teamspeak3.uid, groups)
            self.assertEqual({'Member': 1, 'State': 2}, args[1])  # Check groups
            self.assertEqual(self.member, args[0])  # Check uid

        # Check none user does not have groups updated
        with mock.patch(MODULE_PATH + '.tasks.Teamspeak3Manager') as manager:
            service = self.service()
            none_user = User.objects.get(username=self.none_user)
            service.update_groups(none_user)
            self.assertFalse(manager.return_value.__enter__.return_value.update_user_groups.called)

    @mock.patch(MODULE_PATH + '.tasks.Teamspeak3Manager')
    def test_validate_user(self, manager):
        service = self.service()
        # Test member is not deleted
        member = User.objects.get(username=self.member)
        service.validate_user(member)
        self.assertTrue(member.teamspeak3)

        # Test none user is deleted
        none_user = User.objects.get(username=self.none_user)
        Teamspeak3User.objects.create(user=none_user, uid='abc123', perm_key='132ACB')
        service.validate_user(none_user)
        self.assertTrue(manager.return_value.__enter__.return_value.delete_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            none_teamspeak3 = User.objects.get(username=self.none_user).teamspeak3

    @mock.patch(MODULE_PATH + '.tasks.Teamspeak3Manager')
    def test_delete_user(self, manager):
        member = User.objects.get(username=self.member)

        service = self.service()
        result = service.delete_user(member)

        self.assertTrue(result)
        self.assertTrue(manager.return_value.__enter__.return_value.delete_user.called)
        with self.assertRaises(ObjectDoesNotExist):
            teamspeak3_user = User.objects.get(username=self.member).teamspeak3

    def test_render_services_ctrl(self):
        service = self.service()
        member = User.objects.get(username=self.member)
        request = RequestFactory().get('/services/')
        request.user = member

        response = service.render_services_ctrl(request)
        self.assertTemplateUsed(service.service_ctrl_template)
        self.assertIn(urls.reverse('teamspeak3:deactivate'), response)
        self.assertIn(urls.reverse('teamspeak3:reset_perm'), response)

        # Test register becomes available
        member.teamspeak3.delete()
        member = User.objects.get(username=self.member)
        request.user = member
        response = service.render_services_ctrl(request)
        self.assertIn(urls.reverse('teamspeak3:activate'), response)


class Teamspeak3ViewsTestCase(TestCase):
    def setUp(self):
        # Inert signals before setup begins
        with mock.patch(MODULE_PATH + '.signals.trigger_all_ts_update') as trigger_all_ts_update:
            self.member = AuthUtils.create_member('auth_member')
            self.member.email = 'auth_member@example.com'
            self.member.save()
            AuthUtils.add_main_character(self.member, 'auth_member', '12345', corp_id='111', corp_name='Test Corporation')

            ts_member_group = TSgroup.objects.create(ts_group_id=1, ts_group_name='Member')
            m2m_member = AuthTS.objects.create(auth_group=Group.objects.get(name='Member'))
            m2m_member.ts_group.add(ts_member_group)
            m2m_member.save()
            add_permissions()

    def login(self, user=None, password=None):
        if user is None:
            user = self.member
        self.client.force_login(user)

    @mock.patch(MODULE_PATH + '.forms.Teamspeak3Manager')
    @mock.patch(MODULE_PATH + '.views.Teamspeak3Manager')
    def test_activate(self, manager, forms_manager):
        self.login()
        expected_username = 'auth_member'
        instance = manager.return_value.__enter__.return_value
        instance.add_user.return_value = (expected_username, 'abc123')

        response = self.client.get(urls.reverse('teamspeak3:activate'))

        self.assertTrue(instance.add_user.called)
        teamspeak3_user = Teamspeak3User.objects.get(user=self.member)
        self.assertTrue(teamspeak3_user.uid)
        self.assertTrue(teamspeak3_user.perm_key)
        self.assertRedirects(response, urls.reverse('teamspeak3:verify'), target_status_code=200)

    @mock.patch(MODULE_PATH + '.forms.Teamspeak3Manager')
    @mock.patch(MODULE_PATH + '.tasks.Teamspeak3Manager')
    def test_verify_submit(self, manager, forms_manager):
        self.login()
        expected_username = 'auth_member'

        forms_instance = manager.return_value.__enter__.return_value
        forms_instance._get_userid.return_value = '1234'

        Teamspeak3User.objects.update_or_create(user=self.member, defaults={'uid': '1234', 'perm_key': '5678'})
        data = {'username': 'auth_member'}

        response = self.client.post(urls.reverse('teamspeak3:verify'), data)

        self.assertRedirects(response, urls.reverse('services:services'), target_status_code=200)
        self.assertTrue(manager.return_value.__enter__.return_value.update_groups.called)

    @mock.patch(MODULE_PATH + '.tasks.Teamspeak3Manager')
    def test_deactivate(self, manager):
        self.login()
        Teamspeak3User.objects.create(user=self.member, uid='some member')

        response = self.client.get(urls.reverse('teamspeak3:deactivate'))

        self.assertTrue(manager.return_value.__enter__.return_value.delete_user.called)
        self.assertRedirects(response, expected_url=urls.reverse('services:services'), target_status_code=200)
        with self.assertRaises(ObjectDoesNotExist):
            teamspeak3_user = User.objects.get(pk=self.member.pk).teamspeak3

    @mock.patch(MODULE_PATH + '.tasks.Teamspeak3Manager')
    @mock.patch(MODULE_PATH + '.views.Teamspeak3Manager')
    def test_reset_perm(self, manager, tasks_manager):
        self.login()
        Teamspeak3User.objects.create(user=self.member, uid='some member')

        manager.return_value.__enter__.return_value.generate_new_permissionkey.return_value = "valid_member", "123abc"

        response = self.client.get(urls.reverse('teamspeak3:reset_perm'))

        self.assertRedirects(response, urls.reverse('services:services'), target_status_code=200)
        ts3_user = Teamspeak3User.objects.get(uid='valid_member')
        self.assertEqual(ts3_user.uid, 'valid_member')
        self.assertEqual(ts3_user.perm_key, '123abc')
        self.assertTrue(tasks_manager.return_value.__enter__.return_value.update_groups.called)

    @mock.patch(MODULE_PATH + '.views.Teamspeak3Tasks')
    @mock.patch(MODULE_PATH + '.views.messages')
    def test_should_update_ts_groups(self, messages, Teamspeak3Tasks):
        # given
        self.member.is_superuser = True
        self.member.is_staff = True
        self.member.save()
        self.login()
        # when
        response = self.client.get(urls.reverse('teamspeak3:admin_update_ts3_groups'))
        # then
        self.assertRedirects(
            response, urls.reverse('admin:teamspeak3_authts_changelist'),
            target_status_code=200
        )
        self.assertTrue(messages.info.called)
        self.assertTrue(Teamspeak3Tasks.run_ts3_group_update.delay.called)


class Teamspeak3SignalsTestCase(TestCase):
    def setUp(self):
        self.member = AuthUtils.create_member('auth_member')

        # Suppress signals action while setting up
        with mock.patch(MODULE_PATH + '.signals.trigger_all_ts_update') as trigger_all_ts_update:
            ts_member_group = TSgroup.objects.create(ts_group_id=1, ts_group_name='Member')
            self.m2m_member = AuthTS.objects.create(auth_group=Group.objects.get(name='Member'))
            self.m2m_member.ts_group.add(ts_member_group)
            self.m2m_member.save()

    def test_m2m_signal_registry(self):
        """
        Test that the m2m signal has been registered
        """
        registered_functions = [r[1]() for r in signals.m2m_changed.receivers]
        self.assertIn(m2m_changed_authts_group, registered_functions)

    def test_post_save_signal_registry(self):
        """
        Test that the post_save signal has been registered
        """
        registered_functions = [r[1]() for r in signals.post_save.receivers]
        self.assertIn(post_save_authts, registered_functions)

    def test_post_delete_signal_registry(self):
        """
        Test that the post_delete signal has been registered
        """
        registered_functions = [r[1]() for r in signals.post_delete.receivers]
        self.assertIn(post_delete_authts, registered_functions)

    @mock.patch(MODULE_PATH + '.signals.transaction')
    @mock.patch(MODULE_PATH + '.signals.trigger_all_ts_update')
    def test_m2m_changed_authts_group(self, trigger_all_ts_update, transaction):

        # Overload transaction.on_commit so everything happens synchronously
        transaction.on_commit = lambda fn: fn()

        new_group = TSgroup.objects.create(ts_group_id=99, ts_group_name='new TS group')
        self.m2m_member.ts_group.add(new_group)
        self.m2m_member.save()  # Triggers signal

        self.assertTrue(trigger_all_ts_update.called)

    @mock.patch(MODULE_PATH + '.signals.transaction')
    @mock.patch(MODULE_PATH + '.signals.trigger_all_ts_update')
    def test_post_save_authts(self, trigger_all_ts_update, transaction):

        # Overload transaction.on_commit so everything happens synchronously
        transaction.on_commit = lambda fn: fn()

        AuthTS.objects.create(auth_group=Group.objects.create(name='Test Group'))  # Trigger signal (AuthTS creation)

        self.assertTrue(trigger_all_ts_update.called)

    @mock.patch(MODULE_PATH + '.signals.transaction')
    @mock.patch(MODULE_PATH + '.signals.trigger_all_ts_update')
    def test_post_delete_authts(self, trigger_all_ts_update, transaction):
        # Overload transaction.on_commit so everything happens synchronously
        transaction.on_commit = lambda fn: fn()

        self.m2m_member.delete()  # Trigger delete signal

        self.assertTrue(trigger_all_ts_update.called)

    @mock.patch(MODULE_PATH + '.signals.transaction')
    @mock.patch(MODULE_PATH + '.signals.Teamspeak3Tasks.update_groups.delay')
    def test_state_changed(self, update_groups, transaction):
        # Overload transaction.on_commit so everything happens synchronously
        transaction.on_commit = lambda fn: fn()

        state = AuthUtils.create_state('test', 1000, disconnect_signals=True)
        self.member.profile.state = state
        self.member.profile.save()

        self.assertTrue(update_groups.called)


class Teamspeak3ManagerTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.reserved = ReservedGroupName.objects.create(name='reserved', reason='tests', created_by='Bob, praise be!')

    @staticmethod
    def my_side_effect(*args, **kwargs):
        raise TeamspeakError(1)

    @mock.patch.object(Teamspeak3Manager, '_group_list')
    @mock.patch.object(Teamspeak3Manager, '_group_id_by_name')
    def test_add_user_exception(self, _group_id_by_name, _group_list):
        """test 1st exception occuring in add_user()"""
        # set mocks in Teamspeak3Manager class
        _group_list.return_value = ['Member', 'Guest']
        _group_id_by_name.return_value =  99
        manager = Teamspeak3Manager()
        server = mock.MagicMock()
        server._connected.return_value = True
        server.send_command = mock.Mock(side_effect=Teamspeak3ManagerTestCase.my_side_effect)
        manager._server = server

        # create test data
        user = AuthUtils.create_user("dummy")
        AuthUtils.assign_state(user, AuthUtils.get_member_state())

        # perform test
        manager.add_user(user, "Dummy User")

    @mock.patch.object(Teamspeak3Manager, '_get_userid')
    @mock.patch.object(Teamspeak3Manager, '_user_group_list')
    @mock.patch.object(Teamspeak3Manager, '_add_user_to_group')
    @mock.patch.object(Teamspeak3Manager, '_remove_user_from_group')
    def test_update_groups_add(self, remove, add, groups, userid):
        """Add to one group"""
        userid.return_value = 1
        groups.return_value = {'test': 1}

        Teamspeak3Manager().update_groups(1, {'test': 1, 'dummy': 2})
        self.assertEqual(add.call_count, 1)
        self.assertEqual(remove.call_count, 0)
        self.assertEqual(add.call_args[0][1], 2)

    @mock.patch.object(Teamspeak3Manager, '_get_userid')
    @mock.patch.object(Teamspeak3Manager, '_user_group_list')
    @mock.patch.object(Teamspeak3Manager, '_add_user_to_group')
    @mock.patch.object(Teamspeak3Manager, '_remove_user_from_group')
    def test_update_groups_remove(self, remove, add, groups, userid):
        """Remove from one group"""
        userid.return_value = 1
        groups.return_value = {'test': '1', 'dummy': '2'}

        Teamspeak3Manager().update_groups(1, {'test': 1})
        self.assertEqual(add.call_count, 0)
        self.assertEqual(remove.call_count, 1)
        self.assertEqual(remove.call_args[0][1], 2)

    @mock.patch.object(Teamspeak3Manager, '_get_userid')
    @mock.patch.object(Teamspeak3Manager, '_user_group_list')
    @mock.patch.object(Teamspeak3Manager, '_add_user_to_group')
    @mock.patch.object(Teamspeak3Manager, '_remove_user_from_group')
    def test_update_groups_remove_reserved(self, remove, add, groups, userid):
        """Remove from one group, but do not touch reserved group"""
        userid.return_value = 1
        groups.return_value = {'test': 1, 'dummy': 2, self.reserved.name: 3}

        Teamspeak3Manager().update_groups(1, {'test': 1})
        self.assertEqual(add.call_count, 0)
        self.assertEqual(remove.call_count, 1)
        self.assertEqual(remove.call_args[0][1], 2)

    @mock.patch.object(Teamspeak3Manager, '_group_list')
    def test_sync_group_db_create(self, group_list):
        """Populate the list of all TSgroups"""
        group_list.return_value = {'allowed':'1', 'also allowed':'2'}
        Teamspeak3Manager()._sync_ts_group_db()
        self.assertEqual(TSgroup.objects.all().count(), 2)

    @mock.patch.object(Teamspeak3Manager, '_group_list')
    def test_sync_group_db_delete(self, group_list):
        """Populate the list of all TSgroups, and delete one which no longer exists"""
        TSgroup.objects.create(ts_group_name='deleted', ts_group_id=3)
        group_list.return_value = {'allowed': '1'}
        Teamspeak3Manager()._sync_ts_group_db()
        self.assertEqual(TSgroup.objects.all().count(), 1)
        self.assertFalse(TSgroup.objects.filter(ts_group_name='deleted').exists())

    @mock.patch.object(Teamspeak3Manager, '_group_list')
    def test_sync_group_db_dont_create_reserved(self, group_list):
        """Populate the list of all TSgroups, ignoring a reserved group name"""
        group_list.return_value = {'allowed': '1', 'reserved': '4'}
        Teamspeak3Manager()._sync_ts_group_db()
        self.assertEqual(TSgroup.objects.all().count(), 1)
        self.assertFalse(TSgroup.objects.filter(ts_group_name='reserved').exists())

    @mock.patch.object(Teamspeak3Manager, '_group_list')
    def test_sync_group_db_delete_reserved(self, group_list):
        """Populate the list of all TSgroups, deleting the TSgroup model for one which has become reserved"""
        TSgroup.objects.create(ts_group_name='reserved', ts_group_id=4)
        group_list.return_value = {'allowed': '1', 'reserved': '4'}
        Teamspeak3Manager()._sync_ts_group_db()
        self.assertEqual(TSgroup.objects.all().count(), 1)
        self.assertFalse(TSgroup.objects.filter(ts_group_name='reserved').exists())

    @mock.patch.object(Teamspeak3Manager, '_group_list')
    def test_sync_group_db_partial_addition(self, group_list):
        """Some TSgroups already exist in database, add new ones"""
        TSgroup.objects.create(ts_group_name='allowed', ts_group_id=1)
        group_list.return_value = {'allowed': '1', 'also allowed': '2'}
        Teamspeak3Manager()._sync_ts_group_db()
        self.assertEqual(TSgroup.objects.all().count(), 2)

    @mock.patch.object(Teamspeak3Manager, '_group_list')
    def test_sync_group_db_partial_removal(self, group_list):
        """One TSgroup has been deleted on server, so remove its model"""
        TSgroup.objects.create(ts_group_name='allowed', ts_group_id=1)
        TSgroup.objects.create(ts_group_name='also allowed', ts_group_id=2)
        group_list.return_value = {'allowed': '1'}
        Teamspeak3Manager()._sync_ts_group_db()
        self.assertEqual(TSgroup.objects.all().count(), 1)


class MockRequest:
    pass


class MockSuperUser:
    def has_perm(self, perm, obj=None):
        return True


request = MockRequest()
request.user = MockSuperUser()


class Teamspeak3AdminTestCase(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.site = AdminSite()
        cls.admin = AuthTSgroupAdmin(AuthTS, cls.site)
        cls.group = Group.objects.create(name='test')
        cls.ts_group = TSgroup.objects.create(ts_group_name='test')

    def test_field_queryset_no_reserved_names(self):
        """Ensure all groups are listed when no reserved names"""
        form = self.admin.get_form(request)
        self.assertQuerysetEqual(form.base_fields['auth_group']._get_queryset(), Group.objects.all())
        self.assertQuerysetEqual(form.base_fields['ts_group']._get_queryset(), TSgroup.objects.all())

    def test_field_queryset_reserved_names(self):
        """Ensure reserved group names are filtered out"""
        ReservedGroupName.objects.bulk_create([ReservedGroupName(name='test', reason='tests', created_by='Bob')])
        form = self.admin.get_form(request)
        self.assertQuerysetEqual(form.base_fields['auth_group']._get_queryset(), Group.objects.none())
        self.assertQuerysetEqual(form.base_fields['ts_group']._get_queryset(), TSgroup.objects.none())



from django.contrib.auth.models import User
from django.db import models
from django.utils.translation import gettext as _

from allianceauth.eveonline.models import EveCharacter
from allianceauth.eveonline.models import EveCorporationInfo


class TimerType(models.TextChoices):
    """
    Choices for Timer Type
    """

    UNSPECIFIED = "UNSPECIFIED", _("Not Specified")
    SHIELD = "SHIELD", _("Shield")
    ARMOR = "ARMOR", _("Armor")
    HULL = "HULL", _("Hull")
    FINAL = "FINAL", _("Final")
    ANCHORING = "ANCHORING", _("Anchoring")
    UNANCHORING = "UNANCHORING", _("Unanchoring")


class Timer(models.Model):
    class Meta:
        ordering = ['eve_time']

    details = models.CharField(max_length=254, default="")
    system = models.CharField(max_length=254, default="")
    planet_moon = models.CharField(max_length=254, default="")
    structure = models.CharField(max_length=254, default="")
    timer_type = models.CharField(
        max_length=254,
        choices=TimerType.choices,
        default=TimerType.UNSPECIFIED,
    )
    objective = models.CharField(max_length=254, default="")
    eve_time = models.DateTimeField()
    important = models.BooleanField(default=False)
    eve_character = models.ForeignKey(EveCharacter, null=True, on_delete=models.SET_NULL)
    eve_corp = models.ForeignKey(EveCorporationInfo, on_delete=models.CASCADE)
    corp_timer = models.BooleanField(default=False)
    user = models.ForeignKey(User, null=True, on_delete=models.SET_NULL)

    def __str__(self):
        return str(self.system) + ' ' + str(self.details)



import datetime
import logging

from django.contrib import messages
from django.shortcuts import get_object_or_404
from django.shortcuts import render, redirect
from django.views import View
from django.urls import reverse_lazy
from django.views.generic import CreateView, UpdateView, DeleteView
from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from .form import TimerForm
from .models import Timer

logger = logging.getLogger(__name__)


class BaseTimerView(LoginRequiredMixin, PermissionRequiredMixin, View):
    pass


class TimerView(BaseTimerView):
    template_name = 'timerboard/view.html'
    permission_required = 'auth.timer_view'

    def get(self, request):
        logger.debug(f"timer_view called by user {request.user}")
        char = request.user.profile.main_character
        if char:
            corp = char.corporation
        else:
            corp = None
        base_query = Timer.objects.select_related('eve_character')
        render_items = {
            'timers': base_query.filter(corp_timer=False),
            'corp_timers': base_query.filter(corp_timer=True, eve_corp=corp),
            'future_timers': base_query.filter(corp_timer=False, eve_time__gte=timezone.now()),
            'past_timers': base_query.filter(corp_timer=False, eve_time__lt=timezone.now()),
        }

        return render(request, self.template_name, context=render_items)


class TimerManagementView(BaseTimerView):
    permission_required = 'auth.timer_management'
    index_redirect = 'timerboard:view'
    success_url = reverse_lazy(index_redirect)
    model = Timer
    form_class = TimerForm

    def get_timer(self, timer_id):
        return get_object_or_404(self.model, id=timer_id)


class AddUpdateMixin:
    def get_form_kwargs(self):
        """
        Inject the request user into the kwargs passed to the form
        """
        kwargs = super().get_form_kwargs()
        kwargs.update({'user': self.request.user})
        return kwargs


class AddTimerView(TimerManagementView, AddUpdateMixin, CreateView):
    template_name_suffix = '_create_form'

    def form_valid(self, form):
        result = super().form_valid(form)
        timer = self.object
        logger.info(f"Created new timer in {timer.system} at {timer.eve_time} by user {self.request.user}")
        messages.success(self.request, _('Added new timer in %(system)s at %(time)s.') % {"system": timer.system, "time": timer.eve_time})
        return result


class EditTimerView(TimerManagementView, AddUpdateMixin, UpdateView):
    template_name_suffix = '_update_form'

    def form_valid(self, form):
        messages.success(self.request, _('Saved changes to the timer.'))
        return super().form_valid(form)


class RemoveTimerView(TimerManagementView, DeleteView):
    pass



from django.db import models


class XenforoUser(models.Model):
    user = models.OneToOneField('auth.User',
                                primary_key=True,
                                on_delete=models.CASCADE,
                                related_name='xenforo')
    username = models.CharField(max_length=254)

    def __str__(self):
        return self.username

    class Meta:
        permissions = (
            ("access_xenforo", "Can access the XenForo service"),
        )



"""
DO NOT EDIT THIS FILE

This settings file contains everything needed for Alliance Auth projects to function.
It gets overwritten by the 'allianceauth update' command.
If you wish to make changes, overload the setting in your project's settings file (local.py).
"""

import os

from django.contrib import messages
from celery.schedules import crontab

INSTALLED_APPS = [
    'allianceauth',  # needs to be on top of this list to support favicons in Django admin (see https://gitlab.com/allianceauth/allianceauth/-/issues/1301)
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.humanize',
    'django_celery_beat',
    'bootstrapform',
    'sortedm2m',
    'esi',
    'allianceauth.authentication',
    'allianceauth.services',
    'allianceauth.eveonline',
    'allianceauth.groupmanagement',
    'allianceauth.notifications',
    'allianceauth.thirdparty.navhelper',
    'allianceauth.analytics',
]

SECRET_KEY = "wow I'm a really bad default secret key"

# Celery configuration
BROKER_URL = 'redis://localhost:6379/0'
CELERYBEAT_SCHEDULER = "django_celery_beat.schedulers.DatabaseScheduler"
CELERYBEAT_SCHEDULE = {
    'esi_cleanup_callbackredirect': {
        'task': 'esi.tasks.cleanup_callbackredirect',
        'schedule': crontab(minute=0, hour='*/4'),
    },
    'esi_cleanup_token': {
        'task': 'esi.tasks.cleanup_token',
        'schedule': crontab(minute=0, hour=0),
    },
    'run_model_update': {
        'task': 'allianceauth.eveonline.tasks.run_model_update',
        'schedule': crontab(minute=0, hour="*/6"),
    },
    'check_all_character_ownership': {
        'task': 'allianceauth.authentication.tasks.check_all_character_ownership',
        'schedule': crontab(minute=0, hour='*/4'),
    },
    'analytics_daily_stats': {
        'task': 'allianceauth.analytics.tasks.analytics_daily_stats',
        'schedule': crontab(minute=0, hour=2),
    }
}

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
PROJECT_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
BASE_DIR = os.path.dirname(PROJECT_DIR)

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'allianceauth.analytics.middleware.AnalyticsMiddleware',
]

ROOT_URLCONF = 'allianceauth.urls'

LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locale/'),
)

ugettext = lambda s: s
LANGUAGES = (
    ('en', ugettext('English')),
    ('de', ugettext('German')),
    ('es', ugettext('Spanish')),
    ('zh-hans', ugettext('Chinese Simplified')),
    ('ru', ugettext('Russian')),
    ('ko', ugettext('Korean')),
    ('fr', ugettext('French')),
    ('ja', ugettext('Japanese')),
    ('it', ugettext('Italian')),
)

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(PROJECT_DIR, 'templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'allianceauth.context_processors.auth_settings',
            ],
        },
    },
]

WSGI_APPLICATION = 'allianceauth.wsgi.application'

# Password validation
# https://docs.djangoproject.com/en/1.10/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

AUTHENTICATION_BACKENDS = [
    'allianceauth.authentication.backends.StateBackend',
    'django.contrib.auth.backends.ModelBackend'
]

# Internationalization
# https://docs.djangoproject.com/en/1.10/topics/i18n/

LANGUAGE_CODE = 'en-us'

LANGUAGE_COOKIE_AGE = 1209600

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.10/howto/static-files/
STATIC_URL = '/static/'
STATICFILES_DIRS = [
    os.path.join(PROJECT_DIR, 'static'),
]
STATIC_ROOT = os.path.join(BASE_DIR, 'static')

# Bootstrap messaging css workaround
MESSAGE_TAGS = {
    messages.ERROR: 'danger error'
}

CACHES = {
    "default": {
        "BACKEND": "redis_cache.RedisCache",
        "LOCATION": "localhost:6379",
        "OPTIONS": {
            "DB": 1,
        }
    }
}

SESSION_ENGINE = "django.contrib.sessions.backends.cached_db"

DEBUG = True
ALLOWED_HOSTS = ['*']
DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': str(os.path.join(BASE_DIR, 'alliance_auth.sqlite3')),
    },
}

SITE_NAME = 'Alliance Auth'

LOGIN_URL = 'auth_login_user'  # view that handles login logic

LOGIN_REDIRECT_URL = 'authentication:dashboard'  # default destination when logging in if no redirect specified
LOGOUT_REDIRECT_URL = 'authentication:dashboard'  # destination after logging out
# Both of these redirects accept values as per the django redirect shortcut
# https://docs.djangoproject.com/en/1.11/topics/http/shortcuts/#redirect
# - url names eg 'authentication:dashboard'
# - relative urls eg '/dashboard'
# - absolute urls eg 'http://example.com/dashboard'

# scopes required on new tokens when logging in. Cannot be blank.
LOGIN_TOKEN_SCOPES = ['publicData']

# number of days email verification links are valid for
ACCOUNT_ACTIVATION_DAYS = 1

ESI_API_URL = 'https://esi.evetech.net/'

LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'verbose': {
            'format': "[%(asctime)s] %(levelname)s [%(name)s:%(lineno)s] %(message)s",
            'datefmt': "%d/%b/%Y %H:%M:%S"
        },
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'handlers': {
        'log_file': {
            'level': 'INFO',  # edit this line to change logging level to file
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': os.path.join(BASE_DIR, 'log/allianceauth.log'),
            'formatter': 'verbose',
            'maxBytes': 1024 * 1024 * 5,  # edit this line to change max log file size
            'backupCount': 5,  # edit this line to change number of log backups
        },
        'extension_file': {
            'level': 'INFO',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': os.path.join(BASE_DIR, 'log/extensions.log'),
            'formatter': 'verbose',
            'maxBytes': 1024 * 1024 * 5,  # edit this line to change max log file size
            'backupCount': 5,  # edit this line to change number of log backups
        },
        'console': {
            'level': 'DEBUG',  # edit this line to change logging level to console
            'class': 'logging.StreamHandler',
            'formatter': 'verbose',
        },
        'notifications': {  # creates notifications for users with logging_notifications permission
            'level': 'ERROR',  # edit this line to change logging level to notifications
            'class': 'allianceauth.notifications.handlers.NotificationHandler',
            'formatter': 'verbose',
        },
    },
    'loggers': {
        'allianceauth': {
            'handlers': ['log_file', 'console', 'notifications'],
            'level': 'DEBUG',
        },
        'extensions': {
            'handlers': ['extension_file', 'console'],
            'level': 'DEBUG',
        },
        'django': {
            'handlers': ['log_file', 'console'],
            'level': 'ERROR',
        },
        'esi': {
            'handlers': ['log_file', 'console'],
            'level': 'DEBUG',
        },
    }
}

DEFAULT_AUTO_FIELD = "django.db.models.AutoField"



from datetime import datetime

from django.db import models
from django.utils import timezone

from allianceauth.eveonline.models import EveCharacter


class OpTimerType(models.Model):
    """
    Optimer Type
    """

    type = models.CharField(max_length=254, default="")

    def __str__(self):
        return self.type

    class Meta:
        ordering = ['type']
        default_permissions = ()


class OpTimer(models.Model):
    class Meta:
        ordering = ['start']
        default_permissions = ()

    doctrine = models.CharField(max_length=254, default="")
    system = models.CharField(max_length=254, default="")
    start = models.DateTimeField(default=datetime.now)
    duration = models.CharField(max_length=25, default="")
    operation_name = models.CharField(max_length=254, default="")
    fc = models.CharField(max_length=254, default="")
    post_time = models.DateTimeField(default=timezone.now)
    eve_character = models.ForeignKey(EveCharacter, null=True,
                                      on_delete=models.SET_NULL)
    description = models.TextField(blank=True, default="")
    type = models.ForeignKey(OpTimerType, null=True, on_delete=models.SET_NULL)

    def __str__(self):
        return self.operation_name



import logging
from celery.signals import task_failure, task_success
from django.conf import settings
from allianceauth.analytics.tasks import analytics_event

logger = logging.getLogger(__name__)


@task_failure.connect
def process_failure_signal(
                        exception, traceback,
                        sender, task_id, signal,
                        args, kwargs, einfo, **kw):
    logger.debug("Celery task_failure signal %s" % sender.__class__.__name__)
    if getattr(settings, "ANALYTICS_DISABLED", False):
        return

    category = sender.__module__

    if 'allianceauth.analytics' not in category:
        if category.endswith(".tasks"):
            category = category[:-6]

        action = sender.__name__

        label = f"{exception.__class__.__name__}"

        analytics_event(category=category,
                        action=action,
                        label=label)


@task_success.connect
def celery_success_signal(sender, result=None, **kw):
    logger.debug("Celery task_success signal %s" % sender.__class__.__name__)
    if getattr(settings, "ANALYTICS_DISABLED", False):
        return

    category = sender.__module__

    if 'allianceauth.analytics' not in category:
        if category.endswith(".tasks"):
            category = category[:-6]

        action = sender.__name__
        label = "Success"

        value = 0
        if isinstance(result, int):
            value = result

        analytics_event(category=category,
                        action=action,
                        label=label,
                        value=value)



from django.conf.urls import include, url
from allianceauth import urls

urlpatterns = [
    url(r'', include(urls)),
]

handler500 = 'allianceauth.views.Generic500Redirect'
handler404 = 'allianceauth.views.Generic404Redirect'
handler403 = 'allianceauth.views.Generic403Redirect'
handler400 = 'allianceauth.views.Generic400Redirect'



# Generated by Django 3.2.9 on 2021-11-11 15:56

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('auth', '0012_alter_user_first_name_max_length'),
        ('authentication', '0019_merge_20211026_0919'),
        ('groupmanagement', '0016_remove_grouprequest_status_field'),
    ]

    operations = [
        migrations.AlterField(
            model_name='authgroup',
            name='group_leader_groups',
            field=models.ManyToManyField(blank=True, help_text='Members of leader groups can process requests for this group. Use the <code>auth.group_management</code> permission to allow a user to manage all groups.<br>', related_name='leads_group_groups', to='auth.Group'),
        ),
        migrations.AlterField(
            model_name='authgroup',
            name='group_leaders',
            field=models.ManyToManyField(blank=True, help_text='Group leaders can process requests for this group. Use the <code>auth.group_management</code> permission to allow a user to manage all groups.<br>', related_name='leads_groups', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='authgroup',
            name='open',
            field=models.BooleanField(default=False, help_text='Group is open and users will be automatically added upon request.<br>If the group is not open users will need their request manually approved.'),
        ),
        migrations.AlterField(
            model_name='authgroup',
            name='public',
            field=models.BooleanField(default=False, help_text='Group is public. Any registered user is able to join this group, with visibility based on the other options set for this group.<br>Auth will not remove users from this group automatically when they are no longer authenticated.'),
        ),
        migrations.AlterField(
            model_name='authgroup',
            name='states',
            field=models.ManyToManyField(blank=True, help_text='States listed here will have the ability to join this group provided they have the proper permissions.<br>', related_name='valid_states', to='authentication.State'),
        ),
    ]



# Generated by Django 3.1.13 on 2021-10-15 05:02

from django.core.exceptions import ObjectDoesNotExist
from django.db import migrations


def modify_aa_team_token_add_page_ignore_paths(apps, schema_editor):
    # Add /admin/ and /user_notifications_count/ path to ignore

    AnalyticsPath = apps.get_model('analytics', 'AnalyticsPath')
    admin = AnalyticsPath.objects.create(ignore_path=r"^\/admin\/.*")
    user_notifications_count = AnalyticsPath.objects.create(ignore_path=r"^\/user_notifications_count\/.*")

    Tokens = apps.get_model('analytics', 'AnalyticsTokens')
    token = Tokens.objects.get(token="UA-186249766-2")
    token.ignore_paths.add(admin, user_notifications_count)


def undo_modify_aa_team_token_add_page_ignore_paths(apps, schema_editor):
    #
    AnalyticsPath = apps.get_model('analytics', 'AnalyticsPath')
    Tokens = apps.get_model('analytics', 'AnalyticsTokens')

    token = Tokens.objects.get(token="UA-186249766-2")
    try:
        admin = AnalyticsPath.objects.get(ignore_path=r"^\/admin\/.*", analyticstokens=token)
        user_notifications_count = AnalyticsPath.objects.get(ignore_path=r"^\/user_notifications_count\/.*", analyticstokens=token)
        admin.delete()
        user_notifications_count.delete()
    except ObjectDoesNotExist:
        # Its fine if it doesnt exist, we just dont want them building up when re-migrating
        pass


class Migration(migrations.Migration):

    dependencies = [
        ('analytics', '0003_Generate_Identifier'),
    ]

    operations = [migrations.RunPython(modify_aa_team_token_add_page_ignore_paths, undo_modify_aa_team_token_add_page_ignore_paths)
    ]



# Generated by Django 1.10.1 on 2016-09-05 21:38

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='AuthServicesInfo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ipboard_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('ipboard_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('xenforo_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('xenforo_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('forum_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('forum_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('jabber_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('jabber_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('mumble_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('mumble_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('teamspeak3_uid', models.CharField(blank=True, default=b'', max_length=254)),
                ('teamspeak3_perm_key', models.CharField(blank=True, default=b'', max_length=254)),
                ('discord_uid', models.CharField(blank=True, default=b'', max_length=254)),
                ('discourse_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('discourse_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('ips4_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('ips4_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('ips4_id', models.CharField(blank=True, default=b'', max_length=254)),
                ('smf_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('smf_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('market_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('market_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('pathfinder_username', models.CharField(blank=True, default=b'', max_length=254)),
                ('pathfinder_password', models.CharField(blank=True, default=b'', max_length=254)),
                ('main_char_id', models.CharField(blank=True, default=b'', max_length=64)),
                ('is_blue', models.BooleanField(default=False)),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.views.generic.base import View
from django.http import HttpResponseRedirect
from django.shortcuts import redirect
from django.contrib import messages


class NightModeRedirectView(View):
    SESSION_VAR = "NIGHT_MODE"

    def get(self, request, *args, **kwargs):
        request.session[self.SESSION_VAR] = not self.night_mode_state(request)
        return HttpResponseRedirect(request.GET.get("next", "/"))

    @classmethod
    def night_mode_state(cls, request):
        try:
            return request.session.get(cls.SESSION_VAR, False)
        except AttributeError:
            # Session is middleware
            # Sometimes request wont have a session attribute
            return False


def Generic500Redirect(request):
    messages.error(
        request,
        "Auth encountered an error processing your request, please try again. "
        "If the error persists, please contact the administrators. (500 Internal Server Error)",
    )
    return redirect("authentication:dashboard")


def Generic404Redirect(request, exception):
    messages.error(
        request,
        "Page does not exist. If you believe this is in error please contact the administrators. "
        "(404 Page Not Found)",
    )
    return redirect("authentication:dashboard")


def Generic403Redirect(request, exception):
    messages.error(
        request,
        "You do not have permission to access the requested page. "
        "If you believe this is in error please contact the administrators. (403 Permission Denied)",
    )
    return redirect("authentication:dashboard")


def Generic400Redirect(request, exception):
    messages.error(
        request,
        "Auth encountered an error processing your request, please try again. "
        "If the error persists, please contact the administrators. (400 Bad Request)",
    )
    return redirect("authentication:dashboard")



import logging

from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from celery import shared_task
from allianceauth.services.tasks import QueueOnce
from allianceauth.notifications import notify
from allianceauth.services.hooks import NameFormatter
from .manager import SmfManager
from .models import SmfUser

logger = logging.getLogger(__name__)


class SmfTasks:
    def __init__(self):
        pass

    @classmethod
    def delete_user(cls, user, notify_user=False):
        if cls.has_account(user):
            logger.debug(f"User {user} has a SMF account {user.smf.username}. Deleting.")
            SmfManager.disable_user(user.smf.username)
            user.smf.delete()
            if notify_user:
                notify(user, "SMF Account Disabled", level='danger')
            return True
        return False

    @staticmethod
    def has_account(user):
        try:
            return user.smf.username != ''
        except ObjectDoesNotExist:
            return False

    @classmethod
    def disable(cls):
        SmfUser.objects.all().delete()

    @staticmethod
    @shared_task(bind=True, name="smf.update_groups", base=QueueOnce)
    def update_groups(self, pk):
        user = User.objects.get(pk=pk)
        logger.debug("Updating smf groups for user %s" % user)
        if SmfTasks.has_account(user):
            groups = [user.profile.state.name]
            for group in user.groups.all():
                groups.append(str(group.name))
            logger.debug(f"Updating user {user} smf groups to {groups}")
            try:
                SmfManager.update_groups(user.smf.username, groups)
            except:
                logger.exception("smf group sync failed for %s, retrying in 10 mins" % user)
                raise self.retry(countdown=60 * 10)
            logger.debug("Updated user %s smf groups." % user)
        else:
            logger.debug("User does not have an smf account")

    @staticmethod
    @shared_task(name="smf.update_all_groups")
    def update_all_groups():
        logger.debug("Updating ALL smf groups")
        for user in SmfUser.objects.exclude(username__exact=''):
            SmfTasks.update_groups.delay(user.user_id)

    @staticmethod
    def get_username(user):
        from .auth_hooks import SmfService
        return NameFormatter(SmfService(), user).format_name()



# Generated by Django 3.1.4 on 2020-12-30 13:11

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AnalyticsIdentifier',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('identifier', models.UUIDField(default=uuid.uuid4, editable=False)),
            ],
        ),
        migrations.CreateModel(
            name='AnalyticsPath',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ignore_path', models.CharField(default='/example/', max_length=254)),
            ],
        ),
        migrations.CreateModel(
            name='AnalyticsTokens',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=254)),
                ('type', models.CharField(choices=[('GA-U', 'Google Analytics Universal'), ('GA-V4', 'Google Analytics V4')], max_length=254)),
                ('token', models.CharField(max_length=254)),
                ('send_page_views', models.BooleanField(default=False)),
                ('send_celery_tasks', models.BooleanField(default=False)),
                ('send_stats', models.BooleanField(default=False)),
                ('ignore_paths', models.ManyToManyField(blank=True, to='analytics.AnalyticsPath')),
            ],
        ),
    ]



import logging
import uuid

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.auth.decorators import permission_required
from django.contrib.humanize.templatetags.humanize import intcomma
from django.http import JsonResponse, Http404
from django.shortcuts import render, redirect, get_object_or_404
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.db.models import Sum
from allianceauth.authentication.decorators import permissions_required
from allianceauth.srp.providers import esi
from allianceauth.notifications import notify
from .form import SrpFleetMainForm
from .form import SrpFleetMainUpdateForm
from .form import SrpFleetUserRequestForm
from .models import SrpFleetMain
from .models import SrpUserRequest
from .managers import SRPManager

logger = logging.getLogger(__name__)


def random_string(string_length=10):
    """Returns a random string of length string_length."""
    random = str(uuid.uuid4())  # Convert UUID format to a Python string.
    random = random.upper()  # Make all characters uppercase.
    random = random.replace("-", "")  # Remove the UUID '-'.
    return random[0:string_length]  # Return the random string.


@login_required
@permission_required('srp.access_srp')
def srp_management(request, all=False):
    logger.debug("srp_management called by user %s" % request.user)
    fleets = SrpFleetMain.objects.select_related('fleet_commander').prefetch_related('srpuserrequest_set').all()
    if not all:
        fleets = fleets.filter(fleet_srp_status="")
    else:
        logger.debug("Returning all SRP requests")
    totalcost = fleets.aggregate(total_cost=Sum('srpuserrequest__srp_total_amount')).get('total_cost', 0)
    context = {"srpfleets": fleets, "totalcost": totalcost}
    return render(request, 'srp/management.html', context=context)

@login_required
@permission_required('srp.access_srp')
def srp_fleet_view(request, fleet_id):
    logger.debug(f"srp_fleet_view called by user {request.user} for fleet id {fleet_id}")
    try:
        fleet_main = SrpFleetMain.objects.get(id=fleet_id)
    except SrpFleetMain.DoesNotExist:
        raise Http404
    context = {"fleet_id": fleet_id, "fleet_status": fleet_main.fleet_srp_status,
                "srpfleetrequests": fleet_main.srpuserrequest_set.select_related('character'),
                "totalcost": fleet_main.total_cost}

    return render(request, 'srp/data.html', context=context)


@login_required
@permissions_required(('auth.srp_management', 'srp.add_srpfleetmain'))
def srp_fleet_add_view(request):
    logger.debug("srp_fleet_add_view called by user %s" % request.user)
    completed = False
    completed_srp_code = ""

    if request.method == 'POST':
        form = SrpFleetMainForm(request.POST)
        logger.debug("Request type POST contains form valid: %s" % form.is_valid())
        if form.is_valid():
            srp_fleet_main = SrpFleetMain()
            srp_fleet_main.fleet_name = form.cleaned_data['fleet_name']
            srp_fleet_main.fleet_doctrine = form.cleaned_data['fleet_doctrine']
            srp_fleet_main.fleet_time = form.cleaned_data['fleet_time']
            srp_fleet_main.fleet_srp_code = random_string(8)
            srp_fleet_main.fleet_commander = request.user.profile.main_character

            srp_fleet_main.save()

            completed = True
            completed_srp_code = srp_fleet_main.fleet_srp_code
            logger.info(f"Created SRP Fleet {srp_fleet_main.fleet_name} by user {request.user}")
            messages.success(request, _('Created SRP fleet %(fleetname)s.') % {"fleetname": srp_fleet_main.fleet_name})

    else:
        logger.debug("Returning blank SrpFleetMainForm")
        form = SrpFleetMainForm()

    render_items = {'form': form, "completed": completed, "completed_srp_code": completed_srp_code}

    return render(request, 'srp/add.html', context=render_items)


@login_required
@permission_required('auth.srp_management')
def srp_fleet_remove(request, fleet_id):
    logger.debug(f"srp_fleet_remove called by user {request.user} for fleet id {fleet_id}")
    srpfleetmain = get_object_or_404(SrpFleetMain, id=fleet_id)
    srpfleetmain.delete()
    logger.info(f"SRP Fleet {srpfleetmain.fleet_name} deleted by user {request.user}")
    messages.success(request, _('Removed SRP fleet %(fleetname)s.') % {"fleetname": srpfleetmain.fleet_name})
    return redirect("srp:management")


@login_required
@permission_required('auth.srp_management')
def srp_fleet_disable(request, fleet_id):
    logger.debug(f"srp_fleet_disable called by user {request.user} for fleet id {fleet_id}")
    srpfleetmain = get_object_or_404(SrpFleetMain, id=fleet_id)
    srpfleetmain.fleet_srp_code = ""
    srpfleetmain.save()
    logger.info(f"SRP Fleet {srpfleetmain.fleet_name} disabled by user {request.user}")
    messages.success(request, _('Disabled SRP fleet %(fleetname)s.') % {"fleetname": srpfleetmain.fleet_name})
    return redirect("srp:management")


@login_required
@permission_required('auth.srp_management')
def srp_fleet_enable(request, fleet_id):
    logger.debug(f"srp_fleet_enable called by user {request.user} for fleet id {fleet_id}")
    srpfleetmain = get_object_or_404(SrpFleetMain, id=fleet_id)
    srpfleetmain.fleet_srp_code = random_string(8)
    srpfleetmain.save()
    logger.info(f"SRP Fleet {srpfleetmain.fleet_name} enable by user {request.user}")
    messages.success(request, _('Enabled SRP fleet %(fleetname)s.') % {"fleetname": srpfleetmain.fleet_name})
    return redirect("srp:management")


@login_required
@permission_required('auth.srp_management')
def srp_fleet_mark_completed(request, fleet_id):
    logger.debug(f"srp_fleet_mark_completed called by user {request.user} for fleet id {fleet_id}")
    srpfleetmain = get_object_or_404(SrpFleetMain, id=fleet_id)
    srpfleetmain.fleet_srp_status = "Completed"
    srpfleetmain.save()
    logger.info(f"Marked SRP Fleet {srpfleetmain.fleet_name} as completed by user {request.user}")
    messages.success(request,
                    _('Marked SRP fleet %(fleetname)s as completed.') % {"fleetname": srpfleetmain.fleet_name})
    return redirect("srp:fleet", fleet_id)


@login_required
@permission_required('auth.srp_management')
def srp_fleet_mark_uncompleted(request, fleet_id):
    logger.debug(f"srp_fleet_mark_uncompleted called by user {request.user} for fleet id {fleet_id}")
    srpfleetmain = get_object_or_404(SrpFleetMain, id=fleet_id)
    srpfleetmain.fleet_srp_status = ""
    srpfleetmain.save()
    logger.info(f"Marked SRP Fleet {fleet_id} as incomplete for user {request.user}")
    messages.success(request,
                    _('Marked SRP fleet %(fleetname)s as incomplete.') % {"fleetname": srpfleetmain.fleet_name})
    return redirect("srp:fleet", fleet_id)


@login_required
@permission_required('srp.access_srp')
def srp_request_view(request, fleet_srp):
    logger.debug(f"srp_request_view called by user {request.user} for fleet srp code {fleet_srp}")

    if SrpFleetMain.objects.filter(fleet_srp_code=fleet_srp).exists() is False:
        logger.error(f"Unable to locate SRP Fleet using code {fleet_srp} for user {request.user}")
        messages.error(request,
                        _('Unable to locate SRP code with ID %(srpfleetid)s') % {"srpfleetid": fleet_srp})
        return redirect("srp:management")

    if request.method == 'POST':
        form = SrpFleetUserRequestForm(request.POST)
        logger.debug("Request type POST contains form valid: %s" % form.is_valid())

        if form.is_valid():
            request_killboard_link = form.cleaned_data['killboard_link']
            killmail_id = SRPManager.get_kill_id(killboard_link=request_killboard_link)

            # check if the killmail_id is already present
            if SrpUserRequest.objects.filter(killboard_link__icontains="/kill/" + killmail_id).exists():
                messages.error(request,
                                _("This kill mail has already been posted."))
                return redirect("srp:management")

            character = request.user.profile.main_character
            srp_fleet_main = SrpFleetMain.objects.get(fleet_srp_code=fleet_srp)
            post_time = timezone.now()

            srp_request = SrpUserRequest()
            srp_request.killboard_link = request_killboard_link
            srp_request.additional_info = form.cleaned_data['additional_info']
            srp_request.character = character
            srp_request.srp_fleet_main = srp_fleet_main

            try:
                srp_kill_link = SRPManager.get_kill_id(srp_request.killboard_link)
                (ship_type_id, ship_value, victim_id) = SRPManager.get_kill_data(srp_kill_link)
            except ValueError:
                logger.debug("User {} Submitted Invalid Killmail Link {} or server could not be reached".format(
                    request.user, srp_request.killboard_link))
                # THIS SHOULD BE IN FORM VALIDATION
                messages.error(request,
                                _("Your SRP request Killmail link is invalid. Please make sure you are using zKillboard."))
                return redirect("srp:management")

            if request.user.character_ownerships.filter(character__character_id=str(victim_id)).exists():
                item_type = esi.client.Universe.get_universe_types_type_id(type_id=ship_type_id).result()
                srp_request.srp_ship_name = item_type['name']
                srp_request.kb_total_loss = ship_value
                srp_request.post_time = post_time
                srp_request.save()
                logger.info("Created SRP Request on behalf of user {} for fleet name {}".format(
                    request.user, srp_fleet_main.fleet_name))
                messages.success(request,
                                _('Submitted SRP request for your %(ship)s.') % {"ship": srp_request.srp_ship_name})
                return redirect("srp:management")
            else:
                messages.error(request,
                                _("Character %(charid)s does not belong to your Auth account. Please add the API key for this character and try again")
                                % {"charid": victim_id})
            return redirect("srp:management")
    else:
        logger.debug("Returning blank SrpFleetUserRequestForm")
        form = SrpFleetUserRequestForm()

    render_items = {'form': form}

    return render(request, 'srp/request.html', context=render_items)


@login_required
@permission_required('auth.srp_management')
def srp_request_remove(request):
    numrequests = len(request.POST) - 1
    logger.debug(f"srp_request_remove called by user {request.user} for {numrequests} srp request id's")
    stored_fleet_view = None
    for srp_request_id in request.POST:
        if numrequests == 0:
            messages.warning(request, _("No SRP requests selected"))
            return redirect("srp:management")
        if srp_request_id == "csrfmiddlewaretoken":
            continue
        if SrpUserRequest.objects.filter(id=srp_request_id).exists():
            srpuserrequest = SrpUserRequest.objects.get(id=srp_request_id)
            stored_fleet_view = srpuserrequest.srp_fleet_main.id
            srpuserrequest.delete()
            logger.info(f"Deleted SRP request id {srp_request_id} for user {request.user}")
    if stored_fleet_view is None:
        logger.error("Unable to delete srp requests for user %s - request matching id not found." % (request.user))
        messages.error(request, _('Unable to locate selected SRP request.'))
        return redirect("srp:management")
    else:
        messages.success(request, _('Deleted %(numrequests)s SRP requests') % {"numrequests": numrequests})
        return redirect("srp:fleet", stored_fleet_view)


@login_required
@permission_required('auth.srp_management')
def srp_request_approve(request):
    numrequests = len(request.POST) - 1
    logger.debug(f"srp_request_approve called by user {request.user} for {numrequests} srp request id's")
    stored_fleet_view = None
    for srp_request_id in request.POST:
        if numrequests == 0:
            messages.warning(request, _("No SRP requests selected"))
            return redirect("srp:management")
        if srp_request_id == "csrfmiddlewaretoken":
            continue
        if SrpUserRequest.objects.filter(id=srp_request_id).exists():
            srpuserrequest = SrpUserRequest.objects.get(id=srp_request_id)
            stored_fleet_view = srpuserrequest.srp_fleet_main.id
            srpuserrequest.srp_status = "Approved"
            if srpuserrequest.srp_total_amount == 0:
                srpuserrequest.srp_total_amount = srpuserrequest.kb_total_loss
            srpuserrequest.save()
            logger.info("Approved SRP request id {} for character {} by user {}".format(
                srp_request_id, srpuserrequest.character, request.user))
            notify(
                srpuserrequest.character.character_ownership.user,
                'SRP Request Approved',
                level='success',
                message='Your SRP request for a {} lost during {} has been approved for {} ISK.'.format(
                    srpuserrequest.srp_ship_name, srpuserrequest.srp_fleet_main.fleet_name,
                    intcomma(srpuserrequest.srp_total_amount))
            )
    if stored_fleet_view is None:
        logger.error("Unable to approve srp request on behalf of user %s - request matching id not found." % (request.user))
        messages.error(request, _('Unable to locate selected SRP request.'))
        return redirect("srp:management")
    else:
        messages.success(request, _('Approved %(numrequests)s SRP requests') % {"numrequests": numrequests})
        return redirect("srp:fleet", stored_fleet_view)


@login_required
@permission_required('auth.srp_management')
def srp_request_reject(request):
    numrequests = len(request.POST) - 1
    logger.debug(f"srp_request_reject called by user {request.user} for {numrequests} srp request id's")
    stored_fleet_view = None
    for srp_request_id in request.POST:
        if numrequests == 0:
            messages.warning(request, _("No SRP requests selected"))
            return redirect("srp:management")
        if srp_request_id == "csrfmiddlewaretoken":
            continue
        if SrpUserRequest.objects.filter(id=srp_request_id).exists():
            srpuserrequest = SrpUserRequest.objects.get(id=srp_request_id)
            stored_fleet_view = srpuserrequest.srp_fleet_main.id
            srpuserrequest.srp_status = "Rejected"
            srpuserrequest.save()
            logger.info("SRP request id {} for character {} rejected by {}".format(
                srp_request_id, srpuserrequest.character, request.user))
            notify(
                srpuserrequest.character.character_ownership.user,
                'SRP Request Rejected',
                level='danger',
                message='Your SRP request for a {} lost during {} has been rejected.'.format(
                    srpuserrequest.srp_ship_name, srpuserrequest.srp_fleet_main.fleet_name)
            )
    if stored_fleet_view is None:
        logger.error("Unable to reject SRP request on behalf of user %s - request matching id not found." % (request.user))
        messages.error(request, _('Unable to locate selected SRP request'))
        return redirect("srp:management")
    else:
        messages.success(request, _('Rejected %(numrequests)s SRP requests.') % {"numrequests": numrequests})
        return redirect("srp:fleet", stored_fleet_view)


@login_required
@permission_required('auth.srp_management')
def srp_request_update_amount(request, fleet_srp_request_id):
    logger.debug("srp_request_update_amount called by user {} for fleet srp request id {}".format(
        request.user, fleet_srp_request_id))

    if SrpUserRequest.objects.filter(id=fleet_srp_request_id).exists() is False:
        logger.error(f"Unable to locate SRP request id {fleet_srp_request_id} for user {request.user}")
        messages.error(request,
                        _('Unable to locate SRP request with ID %(requestid)s') % {"requestid": fleet_srp_request_id})
        return redirect("srp:management")

    srp_request = SrpUserRequest.objects.get(id=fleet_srp_request_id)
    srp_request.srp_total_amount = request.POST['value']
    srp_request.save()
    logger.info("Updated srp request id {} total to {} by user {}".format(
        fleet_srp_request_id, request.POST['value'], request.user))
    return JsonResponse({"success": True, "pk": fleet_srp_request_id, "newValue": request.POST['value']})


@login_required
@permission_required('auth.srp_management')
def srp_fleet_edit_view(request, fleet_id):
    logger.debug(f"srp_fleet_edit_view called by user {request.user} for fleet id {fleet_id}")
    srpfleetmain = get_object_or_404(SrpFleetMain, id=fleet_id)
    if request.method == 'POST':
        form = SrpFleetMainUpdateForm(request.POST)
        logger.debug("Request type POST contains form valid: %s" % form.is_valid())
        if form.is_valid():
            srpfleetmain.fleet_srp_aar_link = form.cleaned_data['fleet_aar_link']
            srpfleetmain.save()
            logger.info(f"User {request.user} edited SRP Fleet {srpfleetmain.fleet_name}")
            messages.success(request,
                            _('Saved changes to SRP fleet %(fleetname)s') % {"fleetname": srpfleetmain.fleet_name})
            return redirect("srp:management")
    else:
        logger.debug("Returning blank SrpFleetMainUpdateForm")
        form = SrpFleetMainUpdateForm()
    return render(request, 'srp/update.html', context={'form': form})



from unittest.mock import patch

from django.test import TestCase, RequestFactory
from django.test.utils import override_settings

from allianceauth.notifications.models import Notification
from allianceauth.tests.auth_utils import AuthUtils

from . import TEST_USER_NAME, TEST_USER_ID, add_permissions_to_members, MODULE_PATH
from ..auth_hooks import DiscordService
from ..discord_client import DiscordClient
from ..models import DiscordUser
from ..utils import set_logger_to_file


logger = set_logger_to_file(MODULE_PATH + '.auth_hooks', __file__)


@override_settings(CELERY_ALWAYS_EAGER=True)
class TestDiscordService(TestCase):

    def setUp(self):
        self.member = AuthUtils.create_member(TEST_USER_NAME)
        DiscordUser.objects.create(
            user=self.member,
            uid=TEST_USER_ID,
            username=TEST_USER_NAME,
            discriminator='1234'
        )
        self.none_member = AuthUtils.create_user('Lex Luther')
        self.service = DiscordService
        add_permissions_to_members()
        self.factory = RequestFactory()
        Notification.objects.all().delete()

    def test_service_enabled(self):
        service = self.service()
        self.assertTrue(service.service_active_for_user(self.member))
        self.assertFalse(service.service_active_for_user(self.none_member))

    @patch(MODULE_PATH + '.tasks.update_all_groups')
    def test_update_all_groups(self, mock_update_all_groups):
        service = self.service()
        service.update_all_groups()
        self.assertTrue(mock_update_all_groups.delay.called)

    @patch(MODULE_PATH + '.tasks.update_groups_bulk')
    def test_update_groups_bulk(self, mock_update_groups_bulk):
        service = self.service()
        service.update_groups_bulk([self.member])
        self.assertTrue(mock_update_groups_bulk.delay.called)

    @patch(MODULE_PATH + '.tasks.update_groups')
    def test_update_groups_for_member(self, mock_update_groups):
        service = self.service()
        service.update_groups(self.member)
        self.assertTrue(mock_update_groups.apply_async.called)

    @patch(MODULE_PATH + '.tasks.update_groups')
    def test_update_groups_for_none_member(self, mock_update_groups):
        service = self.service()
        service.update_groups(self.none_member)
        self.assertFalse(mock_update_groups.apply_async.called)

    @patch(MODULE_PATH + '.models.notify')
    @patch(MODULE_PATH + '.tasks.DiscordUser')
    @patch(MODULE_PATH + '.managers.DiscordClient', spec=DiscordClient)
    def test_validate_user(
        self, mock_DiscordClient, mock_DiscordUser, mock_notify
    ):
        mock_DiscordClient.return_value.remove_guild_member.return_value = True

        # Test member is not deleted
        service = self.service()
        service.validate_user(self.member)
        self.assertTrue(DiscordUser.objects.filter(user=self.member).exists())

        # Test none member is deleted
        DiscordUser.objects.create(user=self.none_member, uid=TEST_USER_ID)
        service.validate_user(self.none_member)
        self.assertFalse(DiscordUser.objects.filter(user=self.none_member).exists())

    @patch(MODULE_PATH + '.tasks.update_nickname')
    def test_sync_nickname(self, mock_update_nickname):
        service = self.service()
        service.sync_nickname(self.member)
        self.assertTrue(mock_update_nickname.apply_async.called)

    @patch(MODULE_PATH + '.tasks.update_nicknames_bulk')
    def test_sync_nicknames_bulk(self, mock_update_nicknames_bulk):
        service = self.service()
        service.sync_nicknames_bulk([self.member])
        self.assertTrue(mock_update_nicknames_bulk.delay.called)

    @patch(MODULE_PATH + '.managers.DiscordClient', spec=DiscordClient)
    def test_delete_user_is_member(self, mock_DiscordClient):
        mock_DiscordClient.return_value.remove_guild_member.return_value = True

        service = self.service()
        service.delete_user(self.member, notify_user=True)

        self.assertTrue(mock_DiscordClient.return_value.remove_guild_member.called)
        self.assertFalse(DiscordUser.objects.filter(user=self.member).exists())
        self.assertTrue(Notification.objects.filter(user=self.member).exists())

    @patch(MODULE_PATH + '.managers.DiscordClient', spec=DiscordClient)
    def test_delete_user_is_not_member(self, mock_DiscordClient):
        mock_DiscordClient.return_value.remove_guild_member.return_value = True

        service = self.service()
        service.delete_user(self.none_member)

        self.assertFalse(mock_DiscordClient.return_value.remove_guild_member.called)

    @patch(MODULE_PATH + '.managers.DiscordClient', spec=DiscordClient)
    def test_render_services_ctrl_with_username(self, mock_DiscordClient):
        service = self.service()
        request = self.factory.get('/services/')
        request.user = self.member

        response = service.render_services_ctrl(request)
        self.assertTemplateUsed(service.service_ctrl_template)
        self.assertIn('/discord/reset/', response)
        self.assertIn('/discord/deactivate/', response)

        # Test register becomes available
        self.member.discord.delete()
        self.member.refresh_from_db()
        request.user = self.member
        response = service.render_services_ctrl(request)
        self.assertIn('/discord/activate/', response)

    @patch(MODULE_PATH + '.managers.DiscordClient', spec=DiscordClient)
    def test_render_services_ctrl_wo_username(self, mock_DiscordClient):
        my_member = AuthUtils.create_member('John Doe')
        DiscordUser.objects.create(user=my_member, uid=111222333)
        service = self.service()
        request = self.factory.get('/services/')
        request.user = my_member

        response = service.render_services_ctrl(request)
        self.assertTemplateUsed(service.service_ctrl_template)
        self.assertIn('/discord/reset/', response)
        self.assertIn('/discord/deactivate/', response)



import os
from esi.clients import EsiClientProvider

from allianceauth import __version__

SWAGGER_SPEC = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'swagger.json')

"""
Swagger spec operations:
get_killmails_killmail_id_killmail_hash
get_universe_types_type_id
"""


esi = EsiClientProvider(
    spec_file=SWAGGER_SPEC,
    app_info_text=("allianceauth v" + __version__)
)



# Generated by Django 1.10.5 on 2017-02-04 06:11

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('groupmanagement', '0004_authgroup'),
    ]

    operations = [
        migrations.AddField(
            model_name='authgroup',
            name='public',
            field=models.BooleanField(default=False, help_text='Group is public. Any registered user is able to join this group, with visibility based on the other options set for this group.<br> Auth will not remove users from this group automatically when they are no longer authenticated.'),
        ),
    ]



# Generated by Django 1.10.5 on 2017-03-22 23:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('notifications', '0002_auto_20160910_1649'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notification',
            name='level',
            field=models.CharField(choices=[('danger', 'CRITICAL'), ('danger', 'ERROR'), ('warning', 'WARN'), ('info', 'INFO'), ('success', 'DEBUG')], max_length=10),
        ),
    ]



# Generated by Django 1.10.5 on 2017-03-26 20:13

from django.db import migrations, models
import django.db.models.deletion
import json


def convert_json_to_members(apps, schema_editor):
    CorpStats = apps.get_model('corputils', 'CorpStats')
    CorpMember = apps.get_model('corputils', 'CorpMember')
    for cs in CorpStats.objects.all():
        members = json.loads(cs._members)
        CorpMember.objects.bulk_create(
            [CorpMember(corpstats=cs, character_id=member_id, character_name=member_name) for member_id, member_name in members.items()]
        )


def convert_members_to_json(apps, schema_editor):
    CorpStats = apps.get_model('corputils', 'CorpStats')
    for cs in CorpStats.objects.all():
        cs._members = json.dumps({m.character_id: m.character_name for m in cs.members.all()})
        cs.save()


class Migration(migrations.Migration):
    dependencies = [
        ('corputils', '0003_granular_permissions'),
    ]

    operations = [
        migrations.CreateModel(
            name='CorpMember',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('character_id', models.PositiveIntegerField()),
                ('character_name', models.CharField(max_length=37)),
            ],
            options={
                'ordering': ['character_name'],
            },
        ),
        migrations.AddField(
            model_name='corpmember',
            name='corpstats',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='members',
                                    to='corputils.CorpStats'),
        ),
        migrations.AlterUniqueTogether(
            name='corpmember',
            unique_together={('corpstats', 'character_id')},
        ),
        migrations.RunPython(convert_json_to_members, convert_members_to_json),
        migrations.RemoveField(
            model_name='corpstats',
            name='_members',
        ),
    ]



import logging

from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.http import JsonResponse
from django.shortcuts import render, get_object_or_404, redirect
from django.utils.translation import gettext_lazy as _

from .models import Notification

logger = logging.getLogger(__name__)


@login_required
def notification_list(request):
    logger.debug("notification_list called by user %s" % request.user)
    notifications_qs = Notification.objects.filter(user=request.user).order_by("-timestamp")
    new_notifs = notifications_qs.filter(viewed=False)
    old_notifs = notifications_qs.filter(viewed=True)
    logger.debug(
        "User %s has %s unread and %s read notifications",
        request.user,
        len(new_notifs),
        len(old_notifs)
    )
    context = {
        'read': old_notifs,
        'unread': new_notifs,
    }
    return render(request, 'notifications/list.html', context)


@login_required
def notification_view(request, notif_id):
    logger.debug(
        "notification_view called by user %s for notif_id %s",
        request.user,
        notif_id
    )
    notif = get_object_or_404(Notification, pk=notif_id)
    if notif.user == request.user:
        logger.debug("Providing notification for user %s", request.user)
        context = {'notif': notif}
        notif.mark_viewed()
        return render(request, 'notifications/view.html', context)
    else:
        logger.warn(
            "User %s not authorized to view notif_id %s belonging to user %s",
            request.user,
            notif_id, notif.user
        )
        messages.error(request, _('You are not authorized to view that notification.'))
        return redirect('notifications:list')


@login_required
def remove_notification(request, notif_id):
    logger.debug(
        "remove notification called by user %s for notif_id %s",
        request.user,
        notif_id
    )
    notif = get_object_or_404(Notification, pk=notif_id)
    if notif.user == request.user:
        if Notification.objects.filter(id=notif_id).exists():
            notif.delete()
            logger.info("Deleting notif id %s by user %s", notif_id, request.user)
            messages.success(request, _('Deleted notification.'))
    else:
        logger.error(
            "Unable to delete notif id %s for user %s - notif matching id not found.",
            notif_id,
            request.user
        )
        messages.error(request, _('Failed to locate notification.'))
    return redirect('notifications:list')


@login_required
def mark_all_read(request):
    logger.debug('mark all notifications read called by user %s', request.user)
    Notification.objects.filter(user=request.user).update(viewed=True)
    messages.success(request, _('Marked all notifications as read.'))
    return redirect('notifications:list')


@login_required
def delete_all_read(request):
    logger.debug('delete all read notifications called by user %s', request.user)
    Notification.objects.filter(user=request.user).filter(viewed=True).delete()
    messages.success(request, _('Deleted all read notifications.'))
    return redirect('notifications:list')


def user_notifications_count(request, user_pk: int):
    """returns to notifications count for the give user as JSON

    This view is public and does not require login
    """
    unread_count = Notification.objects.user_unread_count(user_pk)
    data = {'unread_count': unread_count}
    return JsonResponse(data, safe=False)
